@@ -1,31 +1,39 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from collections import OrderedDict
 from datetime import datetime
 from releng_tool import __version__ as releng_version
 from releng_tool.defs import ConfKey
+from releng_tool.defs import GBL_LSRCS
 from releng_tool.defs import GlobalAction
 from releng_tool.defs import PkgAction
 from releng_tool.defs import VcsType
 from releng_tool.engine.fetch import stage as fetch_stage
 from releng_tool.engine.init import initialize_sample
+from releng_tool.engine.license import LicenseManager
+from releng_tool.engine.sbom import SbomManager
+from releng_tool.exceptions import RelengToolInvalidConfigurationScript
+from releng_tool.exceptions import RelengToolInvalidConfigurationSettings
+from releng_tool.exceptions import RelengToolInvalidOverrideConfigurationScript
 from releng_tool.exceptions import RelengToolMissingConfigurationError
 from releng_tool.exceptions import RelengToolMissingPackagesError
 from releng_tool.opts import RELENG_POST_BUILD_NAME
 from releng_tool.packages.exceptions import RelengToolStageFailure
 from releng_tool.packages.manager import RelengPackageManager
+from releng_tool.packages.pipeline import PipelineResult
 from releng_tool.packages.pipeline import RelengPackagePipeline
 from releng_tool.prerequisites import RelengPrerequisites
 from releng_tool.registry import RelengRegistry
 from releng_tool.stats import RelengStats
 from releng_tool.support import releng_include
 from releng_tool.support import require_version
+from releng_tool.tool.python import PYTHON
 from releng_tool.util.env import env_value
 from releng_tool.util.env import extend_script_env
 from releng_tool.util.file_flags import FileFlag
 from releng_tool.util.file_flags import check_file_flag
-from releng_tool.util.file_flags import process_file_flag
 from releng_tool.util.io import FailedToPrepareWorkingDirectoryError
 from releng_tool.util.io import cat
 from releng_tool.util.io import ensure_dir_exists
@@ -35,14 +43,17 @@ from releng_tool.util.io import generate_temp_dir
 from releng_tool.util.io import interim_working_dir
 from releng_tool.util.io import ls
 from releng_tool.util.io import opt_file
-from releng_tool.util.io import path_copy
 from releng_tool.util.io import path_exists
-from releng_tool.util.io import path_move
 from releng_tool.util.io import path_remove
 from releng_tool.util.io import run_script
 from releng_tool.util.io import touch
+from releng_tool.util.io_copy import path_copy
+from releng_tool.util.io_copy import path_copy_into
+from releng_tool.util.io_move import path_move
+from releng_tool.util.io_move import path_move_into
 from releng_tool.util.log import debug
 from releng_tool.util.log import err
+from releng_tool.util.log import hint
 from releng_tool.util.log import log
 from releng_tool.util.log import note
 from releng_tool.util.log import success
@@ -53,11 +64,15 @@ from releng_tool.util.string import expand
 from releng_tool.util.string import interpret_dictionary_strings
 from releng_tool.util.string import interpret_string
 from releng_tool.util.string import interpret_strings
-from shutil import copyfileobj
+import json
 import os
 import ssl
 import sys
 
+if sys.version_info < (3, 0):
+    import imp
+
+
 class RelengEngine:
     """
     releng tool's engine
@@ -72,6 +87,7 @@ class RelengEngine:
         opts: options used to configure the engine
         pkgman: manager for package-related tasks
         registry: extension registry
+        stats: statistics tracker
     """
     def __init__(self, opts):
         self.registry = RelengRegistry()
@@ -79,6 +95,20 @@ class RelengEngine:
         self.pkgman = RelengPackageManager(opts, self.registry, dvcs_cache=True)
         self.stats = RelengStats(opts)
 
+        # load spdx license data
+        if sys.version_info < (3, 0) and not os.path.isabs(__file__):
+            _, base_dir, _ = imp.find_module('releng_tool')
+        else:
+            engine_dir = os.path.dirname(__file__)
+            base_dir = os.path.dirname(engine_dir)
+
+        data_dir = os.path.join(base_dir, 'data')
+        licenses_file = os.path.join(data_dir, 'licenses', 'data.json')
+
+        debug('loading spdx license database: {}', licenses_file)
+        with open(licenses_file, mode='r') as f:
+            self.opts.spdx = json.load(f)
+
     def run(self):
         """
         run the engine
@@ -94,13 +124,12 @@ class RelengEngine:
 
         opts = self.opts
         gaction = opts.gbl_action
-        license_files = {}
         pa = opts.pkg_action
 
         if gaction == GlobalAction.INIT:
             return initialize_sample(opts)
 
-        start_time = datetime.now().replace(microsecond=0)
+        start_time = datetime.now().replace(microsecond=0)  # noqa: DTZ005
         gbls = {
             'releng_args': opts.forward_args,
             'releng_version': releng_version,
@@ -115,24 +144,35 @@ class RelengEngine:
         debug('loading statistics...')
         self.stats.load()
 
+        # verify the project's configuration exists before performing any
+        # actions
+        verbose('detecting project configuration...')
+        conf_point, conf_point_exists = opt_file(opts.conf_point)
+        if not conf_point_exists:
+            raise RelengToolMissingConfigurationError(conf_point)
+
+        # file flag processing
+        state = self._process_file_flags()
+        if state is not None:
+            debug('file-flag processing has triggered closure')
+            return state
+
         # inform the user of any active running modes
         if self.opts.devmode:
-            verbose('running in development mode')
+            if self.opts.devmode is True:
+                postfix = ''
+            else:
+                postfix = ' ({})'.format(self.opts.devmode)
+            hint('running in development mode' + postfix)
 
         if self.opts.local_srcs:
-            verbose('running in local-sources mode')
+            hint('running in local-sources mode')
 
         # register the project's root directory as a system path; permits a
         # project to import locally created modules in their build/etc. scripts
         debug('registering root directory in path...')
         sys.path.append(opts.root_dir)
-
-        # register the project's host directory as a system path; lazily permits
-        # loading host tools (not following prefix or bin container) built by a
-        # project over the system
-        debug('registering host directory in path...')
-        sys.path.insert(0, opts.host_dir)
-        os.environ['PATH'] = opts.host_dir + os.pathsep + os.environ['PATH']
+        os.environ['PATH'] = os.environ['PATH'] + os.pathsep + opts.root_dir
 
         # prepare script environment to make helpers available to configuration
         # script(s)
@@ -143,14 +183,10 @@ class RelengEngine:
         # was configured). This is peformed later in the engine run call.
         self._prepare_script_environment(gbls, gaction, opts.pkg_action)
 
-        verbose("loading user's configuration...")
-        conf_point, conf_point_exists = opt_file(opts.conf_point)
-        if not conf_point_exists:
-            raise RelengToolMissingConfigurationError(conf_point)
-
+        verbose('loading project configuration...')
         settings = run_script(conf_point, gbls, subject='configuration')
         if not settings:
-            return False
+            raise RelengToolInvalidConfigurationScript
 
         script_env = gbls.copy()
         extend_script_env(script_env, settings)
@@ -164,23 +200,21 @@ class RelengEngine:
             overrides = run_script(opts.conf_point_overrides, gbls,
                 subject='configuration overrides')
             if not overrides:
-                return False
+                raise RelengToolInvalidOverrideConfigurationScript
 
             extend_script_env(script_env, overrides)
             extend_script_env(settings, overrides)
             verbose('configuration overrides file loaded')
 
         # handle cleaning requests
-        if (gaction == GlobalAction.CLEAN or gaction == GlobalAction.MRPROPER
-                or gaction == GlobalAction.DISTCLEAN):
-            self._handle_clean_request(gaction)
-            return True
+        clean_actions = [
+            GlobalAction.CLEAN,
+            GlobalAction.DISTCLEAN,
+            GlobalAction.MRPROPER,
+        ]
 
-        # file flag processing
-        state = self._process_file_flags()
-        if state is not None:
-            debug('file-flag processing has triggered closure')
-            return state
+        if gaction in clean_actions:
+            return self._handle_clean_request(gaction)
 
         # determine package name(s) extraction
         #
@@ -199,15 +233,43 @@ class RelengEngine:
 
         # processing additional settings
         if not self._process_settings(settings):
-            return False
+            raise RelengToolInvalidConfigurationSettings
+
+        # register the project's host directory as a system path; lazily permits
+        # loading host tools (not following prefix or bin container) built by a
+        # project over the system
+        debug('registering host directory in path...')
+        sys.path.insert(0, opts.host_dir)
+        os.environ['PATH'] = opts.host_dir + os.pathsep + os.environ['PATH']
 
         # register the project's host-bin directory as a system path; lazily
         # permits loading host tools built by a project over the system
         debug('registering host bin directory in path...')
-        host_bin_dir = os.path.join(opts.host_dir + opts.sysroot_prefix, 'bin')
+        host_sysroot_dir = os.path.join(opts.host_dir + opts.sysroot_prefix)
+        host_bin_dir = os.path.join(host_sysroot_dir, 'bin')
         sys.path.insert(0, host_bin_dir)
         os.environ['PATH'] = host_bin_dir + os.pathsep + os.environ['PATH']
 
+        # register additional host directories, if python is installed;
+        # although this does not cover a varity of use cases such as custom
+        # interpreter overrides for specific Python package
+        if PYTHON.exists():
+            # include the host environment's site-package folder (if any)
+            debug('registering PYTHONPATH host path...')
+            host_python_path = PYTHON.path(
+                sysroot=opts.host_dir, prefix=opts.sysroot_prefix)
+            if 'PYTHONPATH' in os.environ:
+                host_python_path += os.pathsep + os.environ['PYTHONPATH']
+            os.environ['PYTHONPATH'] = host_python_path
+
+            # if Windows, also register a path to a common scripts folder for
+            # Python installation (if host-based Python packages are built)
+            if sys.platform == 'win32':
+                debug('registering Python-Scripts path...')
+                sdir = os.path.join(host_sysroot_dir, 'Scripts')
+                sys.path.insert(0, sdir)
+                os.environ['PATH'] = sdir + os.pathsep + os.environ['PATH']
+
         # load and process packages
         pkgs = self.pkgman.load(pkg_names)
         if not pkgs:
@@ -218,25 +280,31 @@ class RelengEngine:
             for pkg in pkgs:
                 if pkg.name == opts.target_action:
                     def pkg_verbose_clean(desc):
-                        verbose('{} for package: {}', desc, pkg.name)
+                        pkg_name = pkg.name  # noqa: B023
+                        verbose('{} for package: {}', desc, pkg_name)
                     pkg_verbose_clean('removing output directory')
-                    path_remove(pkg.build_output_dir)
+                    rv = path_remove(pkg.build_output_dir)
 
                     if pa == PkgAction.DISTCLEAN:
                         if os.path.exists(pkg.cache_file):
                             pkg_verbose_clean('removing cache file')
-                            path_remove(pkg.cache_file)
+                            rv &= path_remove(pkg.cache_file)
                         if os.path.isdir(pkg.cache_dir):
                             pkg_verbose_clean('removing cache directory')
-                            path_remove(pkg.cache_dir)
+                            rv &= path_remove(pkg.cache_dir)
 
-                    return True
-            assert False # should not reach here
+                    return rv
+            raise AssertionError('unexpected package clean handling')
 
         # ensure any of required host tools do exist
         if 'releng.disable_prerequisites_check' not in opts.quirks:
+            exclude_host_check = set()
+            for pkg in pkgs:
+                if pkg.host_provides:
+                    exclude_host_check.update(pkg.host_provides)
+
             prerequisites = RelengPrerequisites(pkgs, opts.prerequisites)
-            if not prerequisites.check():
+            if not prerequisites.check(exclude=exclude_host_check):
                 return False
 
         # track if this action is "pre-configuration", where a package
@@ -270,8 +338,7 @@ class RelengEngine:
 
                 # in the event that we are not explicit fetching and the package
                 # has already been extracted, completely skip the fetching stage
-                is_local_pkg = pkg.is_internal and opts.local_srcs
-                if not requested_fetch and not is_local_pkg:
+                if not requested_fetch and not pkg.local_srcs:
                     flag = pkg._ff_extract
                     if check_file_flag(flag) == FileFlag.EXISTS:
                         continue
@@ -320,9 +387,8 @@ class RelengEngine:
                         continue
 
                     verbose('processing package: {}', pkg.name)
-                    if not pipeline.process(pkg):
+                    if pipeline.process(pkg) == PipelineResult.STOP:
                         return True
-                license_files = pipeline.license_files
 
         except RelengToolStageFailure:
             return False
@@ -338,12 +404,21 @@ has failed. Ensure the following path is accessible for this user:
 
         is_action = (gaction or pa or opts.target_action is not None)
 
+        # perform sbom generation
+        if gaction == GlobalAction.SBOM or not is_action:
+            note('generating sbom information...')
+            sbom_manager = SbomManager(opts)
+            sbom_cache = sbom_manager.build_cache(pkgs)
+            if not sbom_manager.generate(sbom_cache):
+                return False
+
         # perform license generation
         if gaction == GlobalAction.LICENSES or pa == PkgAction.LICENSE \
                 or not is_action:
             note('generating license information...')
-
-            if not self._perform_license_generation(license_files):
+            license_manager = LicenseManager(opts)
+            license_cache = license_manager.build_cache(pkgs)
+            if not license_manager.generate(license_cache):
                 return False
 
         # perform post-processing and completion message if not performing a
@@ -353,7 +428,7 @@ has failed. Ensure the following path is accessible for this user:
                 err('failed to perform post-processing')
                 return False
 
-            end_time = datetime.now().replace(microsecond=0)
+            end_time = datetime.now().replace(microsecond=0)  # noqa: DTZ005
             success('completed ({})', end_time - start_time)
 
         # attempt to generate a report at the end of a run
@@ -376,16 +451,18 @@ has failed. Ensure the following path is accessible for this user:
             otherwise
         """
 
+        rv = True
+
         # forced request
         #
         # When a forced request is made, clear all file flags so these stages
         # can be invoked again.
         if self.opts.force:
-            path_remove(pkg._ff_bootstrap)
-            path_remove(pkg._ff_configure)
-            path_remove(pkg._ff_build)
-            path_remove(pkg._ff_install)
-            path_remove(pkg._ff_post)
+            rv &= path_remove(pkg._ff_bootstrap)
+            rv &= path_remove(pkg._ff_configure)
+            rv &= path_remove(pkg._ff_build)
+            rv &= path_remove(pkg._ff_install)
+            rv &= path_remove(pkg._ff_post)
 
         # user invoking a package-specific override
         #
@@ -395,21 +472,21 @@ has failed. Ensure the following path is accessible for this user:
         elif pkg.name == self.opts.target_action:
             if (self.opts.pkg_action in
                     (PkgAction.REBUILD, PkgAction.REBUILD_ONLY)):
-                path_remove(pkg._ff_build)
-                path_remove(pkg._ff_install)
-                path_remove(pkg._ff_post)
+                rv &= path_remove(pkg._ff_build)
+                rv &= path_remove(pkg._ff_install)
+                rv &= path_remove(pkg._ff_post)
             elif (self.opts.pkg_action in
                     (PkgAction.RECONFIGURE, PkgAction.RECONFIGURE_ONLY)):
-                path_remove(pkg._ff_bootstrap)
-                path_remove(pkg._ff_configure)
-                path_remove(pkg._ff_build)
-                path_remove(pkg._ff_install)
-                path_remove(pkg._ff_post)
+                rv &= path_remove(pkg._ff_bootstrap)
+                rv &= path_remove(pkg._ff_configure)
+                rv &= path_remove(pkg._ff_build)
+                rv &= path_remove(pkg._ff_install)
+                rv &= path_remove(pkg._ff_post)
             elif self.opts.pkg_action == PkgAction.REINSTALL:
-                path_remove(pkg._ff_install)
-                path_remove(pkg._ff_post)
+                rv &= path_remove(pkg._ff_install)
+                rv &= path_remove(pkg._ff_post)
 
-        return True
+        return rv
 
     def _get_package_names(self, settings):
         """
@@ -471,91 +548,44 @@ of the releng process:
             gaction: the specific clean action being requested
         """
 
+        rv = True
+
         if gaction == GlobalAction.DISTCLEAN:
             verbose('removing cache directory')
-            path_remove(self.opts.cache_dir)
+            rv &= path_remove(self.opts.cache_dir)
             verbose('removing download directory')
-            path_remove(self.opts.dl_dir)
+            rv &= path_remove(self.opts.dl_dir)
 
         if gaction in (GlobalAction.MRPROPER, GlobalAction.DISTCLEAN):
             verbose('removing output directory')
-            path_remove(self.opts.out_dir)
+            rv &= path_remove(self.opts.out_dir)
 
             verbose('removing file flags')
             if os.path.exists(self.opts.ff_devmode):
                 if path_remove(self.opts.ff_devmode):
                     warn('development mode has been unconfigured')
+                else:
+                    rv = False
             if os.path.exists(self.opts.ff_local_srcs):
                 if path_remove(self.opts.ff_local_srcs):
                     warn('local-sources mode has been unconfigured')
+                else:
+                    rv = False
         else:
             verbose('removing build directory')
-            path_remove(self.opts.build_dir)
+            rv &= path_remove(self.opts.build_dir)
             verbose('removing host directory')
-            path_remove(self.opts.host_dir)
+            rv &= path_remove(self.opts.host_dir)
             verbose('removing license directory')
-            path_remove(self.opts.license_dir)
+            rv &= path_remove(self.opts.license_dir)
             verbose('removing staging directory')
-            path_remove(self.opts.staging_dir)
+            rv &= path_remove(self.opts.staging_dir)
             verbose('removing symbols directory')
-            path_remove(self.opts.symbols_dir)
+            rv &= path_remove(self.opts.symbols_dir)
             verbose('removing target directory')
-            path_remove(self.opts.target_dir)
+            rv &= path_remove(self.opts.target_dir)
 
-    def _perform_license_generation(self, license_files):
-        """
-        generate a license file for the project
-
-        Compiles a document containing all the license information for a
-        configured project. License information defined by each package will be
-        populated into a single file after each package is examined. Returned
-        will be a dictionary where keys are the package names and the values are
-        the list of one or more license files for the package.
-
-        Args:
-            license_files: dictionary of each package's license files
-
-        Returns:
-            ``True`` if the license file was generated; ``False`` if the license
-            file could not be generated
-        """
-        if not ensure_dir_exists(self.opts.license_dir):
-            return False
-
-        license_file = os.path.join(self.opts.license_dir, 'licenses')
-        try:
-            with open(license_file, 'w') as dst:
-                license_header = expand(self.opts.license_header)
-                if not license_header:
-                    license_header = 'license(s)'
-
-                # output license header
-                dst.write('''{}
-################################################################################
-'''.format(license_header))
-
-                # output license header
-                for license_name, license_data in sorted(license_files.items()):
-                    license_files = license_data['files']
-                    license_version = license_data['version']
-                    dst.write('''
-{}-{}
---------------------------------------------------------------------------------
-'''.format(license_name, license_version))
-                    for pkg_license_file in sorted(license_files):
-                        verbose('writing license file ({}): {}',
-                            license_name, pkg_license_file)
-                        with open(pkg_license_file, 'r') as f:
-                            copyfileobj(f, dst)
-                        dst.write('')
-
-            verbose('license file has been written')
-        except IOError as e:
-            err('unable to populate license information\n'
-                '    {}', e)
-            return False
-
-        return True
+        return rv
 
     def _post_processing(self, env):
         """
@@ -632,6 +662,7 @@ of the releng process:
         script_env['RELENG_REBUILD'] = None
         script_env['RELENG_RECONFIGURE'] = None
         script_env['RELENG_REINSTALL'] = None
+        script_env['RELENG_TARGET_PKG'] = None
         script_env['RELENG_VERBOSE'] = None
 
         #: default lib container directory
@@ -639,10 +670,13 @@ of the releng process:
         host_pdir = self.opts.host_dir + sysroot_nprefix
         staging_pdir = self.opts.staging_dir + sysroot_nprefix
         target_pdir = self.opts.target_dir + sysroot_nprefix
+        host_bin_dir = os.path.join(host_pdir, 'bin')
         host_include_dir = os.path.join(host_pdir, 'include')
         host_lib_dir = os.path.join(host_pdir, 'lib')
+        staging_bin_dir = os.path.join(staging_pdir, 'bin')
         staging_include_dir = os.path.join(staging_pdir, 'include')
         staging_lib_dir = os.path.join(staging_pdir, 'lib')
+        target_bin_dir = os.path.join(target_pdir, 'bin')
         target_include_dir = os.path.join(target_pdir, 'include')
         target_lib_dir = os.path.join(target_pdir, 'lib')
 
@@ -651,6 +685,7 @@ of the releng process:
             env['BUILD_DIR'] = self.opts.build_dir
             env['CACHE_DIR'] = self.opts.cache_dir
             env['DL_DIR'] = self.opts.dl_dir
+            env['HOST_BIN_DIR'] = host_bin_dir
             env['HOST_DIR'] = self.opts.host_dir
             env['HOST_INCLUDE_DIR'] = host_include_dir
             env['HOST_LIB_DIR'] = host_lib_dir
@@ -665,22 +700,28 @@ of the releng process:
             env['PREFIXED_TARGET_DIR'] = target_pdir
             env['RELENG_VERSION'] = releng_version
             env['ROOT_DIR'] = self.opts.root_dir
+            env['STAGING_BIN_DIR'] = staging_bin_dir
             env['STAGING_DIR'] = self.opts.staging_dir
             env['STAGING_INCLUDE_DIR'] = staging_include_dir
             env['STAGING_LIB_DIR'] = staging_lib_dir
             env['SYMBOLS_DIR'] = self.opts.symbols_dir
+            env['TARGET_BIN_DIR'] = target_bin_dir
             env['TARGET_DIR'] = self.opts.target_dir
             env['TARGET_INCLUDE_DIR'] = target_include_dir
             env['TARGET_LIB_DIR'] = target_lib_dir
 
-            if gaction == GlobalAction.CLEAN:
+            if self.opts.target_action:
+                env['RELENG_TARGET_PKG'] = self.opts.target_action
+
+            if gaction == GlobalAction.CLEAN or paction == PkgAction.CLEAN:
                 env['RELENG_CLEAN'] = '1'
-            elif gaction == GlobalAction.DISTCLEAN:
-                env['RELENG_CLEAN'] = '1' # also set clean flag
+            elif gaction == GlobalAction.DISTCLEAN or \
+                    paction == PkgAction.DISTCLEAN:
+                env['RELENG_CLEAN'] = '1'  # also set clean flag
                 env['RELENG_DISTCLEAN'] = '1'
-                env['RELENG_MRPROPER'] = '1' # also set mrproper flag
+                env['RELENG_MRPROPER'] = '1'  # also set mrproper flag
             elif gaction == GlobalAction.MRPROPER:
-                env['RELENG_CLEAN'] = '1' # also set clean flag
+                env['RELENG_CLEAN'] = '1'  # also set clean flag
                 env['RELENG_MRPROPER'] = '1'
 
             if paction in (PkgAction.REBUILD, PkgAction.REBUILD_ONLY):
@@ -693,7 +734,10 @@ of the releng process:
             if self.opts.debug:
                 env['RELENG_DEBUG'] = '1'
             if self.opts.devmode:
-                env['RELENG_DEVMODE'] = '1'
+                if self.opts.devmode is True:
+                    env['RELENG_DEVMODE'] = '1'
+                else:
+                    env['RELENG_DEVMODE'] = self.opts.devmode
             if self.opts.force:
                 env['RELENG_FORCE'] = '1'
             if self.opts.local_srcs:
@@ -708,6 +752,7 @@ of the releng process:
         script_env['note'] = note
         script_env['releng_cat'] = cat
         script_env['releng_copy'] = path_copy
+        script_env['releng_copy_into'] = path_copy_into
         script_env['releng_env'] = env_value
         script_env['releng_execute'] = execute
         script_env['releng_execute_rv'] = execute_rv
@@ -719,6 +764,7 @@ of the releng process:
         script_env['releng_ls'] = ls
         script_env['releng_mkdir'] = ensure_dir_exists
         script_env['releng_move'] = path_move
+        script_env['releng_move_into'] = path_move_into
         script_env['releng_remove'] = path_remove
         script_env['releng_require_version'] = require_version
         script_env['releng_tmpdir'] = generate_temp_dir
@@ -746,25 +792,113 @@ of the releng process:
         configured = False
         err_flag = False
 
-        state = process_file_flag(opts.devmode, opts.ff_devmode)
-        if state == FileFlag.CONFIGURED:
-            success('configured root for development mode')
-            configured = True
-        elif state == FileFlag.NOT_CONFIGURED:
-            err_flag = True
-        opts.devmode = (state == FileFlag.EXISTS)
-        if opts.devmode:
-            verbose('development mode enabled')
+        # parse and populate development mode configuration
+        devmode_changed = bool(opts.devmode)
 
-        state = process_file_flag(opts.local_srcs, opts.ff_local_srcs)
-        if state == FileFlag.CONFIGURED:
-            success('configured root for local-sources mode')
-            configured = True
-        elif state == FileFlag.NOT_CONFIGURED:
-            err_flag = True
-        opts.local_srcs = (state == FileFlag.EXISTS)
-        if opts.local_srcs:
-            verbose('local-sources mode enabled')
+        devmode_cfg = {}
+        if os.path.exists(opts.ff_devmode):
+            # if we have an empty file, assume this is an old "file flag"
+            # for development mode -- if we are not in a configured
+            # development mode, enable it now
+            if os.stat(opts.ff_devmode).st_size == 0:
+                if not devmode_changed:
+                    opts.devmode = True
+            else:
+                with open(opts.ff_devmode, mode='r') as f:
+                    try:
+                        devmode_cfg = json.load(f)
+                        if not devmode_changed:
+                            opts.devmode = devmode_cfg['mode']
+                    except Exception as e:
+                        # inform user of error, unless we are overwriting
+                        if not devmode_changed:
+                            err_flag = True
+                            err('''\
+failed to parse development mode file
+
+The file used to track `--development` options cannot be read. It is
+recommended to remove the file manually and reconfigure the environment
+for any desired locally sourced packages.
+
+     File: {}
+    Error: {}''', opts.ff_devmode, e)
+
+        if devmode_changed:
+            devmode_cfg['mode'] = opts.devmode
+
+            try:
+                with open(opts.ff_devmode, 'w') as f:
+                    json.dump(devmode_cfg, f)
+
+                    success('configured root for development mode')
+                    configured = True
+            except Exception as e:
+                err_flag = True
+                err('''\
+failed to write development mode file
+
+The file used to track `--development` options cannot be written to.
+
+     File: {}
+    Error: {}''', opts.ff_devmode, e)
+
+        # parse and populate local sources configurations
+        local_srcs_changed = bool(opts.local_srcs)
+
+        if os.path.exists(opts.ff_local_srcs):
+            # if we have an empty file, assume this is an old "file flag"
+            # local sources configuration
+            if os.stat(opts.ff_local_srcs).st_size == 0:
+                if GBL_LSRCS not in opts.local_srcs:
+                    opts.local_srcs[GBL_LSRCS] = None
+            else:
+                with open(opts.ff_local_srcs, mode='r') as f:
+                    try:
+                        # cache any new local sources configurations, pull in
+                        # the configure local sources content and apply any new
+                        # configuration to the options
+                        new_local_srcs = opts.local_srcs
+                        opts.local_srcs = json.load(f)
+                        opts.local_srcs.update(new_local_srcs)
+
+                    # if we failed to load the file, assume this is an old
+                    # "file flag" local sources configuration
+                    except Exception as e:
+                        # inform user of error, unless we are overwriting
+                        if not local_srcs_changed:
+                            err_flag = True
+                            err('''\
+failed to parse local sources file
+
+The file used to track `--local-sources` options cannot be read. It is
+recommended to remove the file manually and reconfigure the environment
+for any desired locally sourced packages.
+
+     File: {}
+    Error: {}''', opts.ff_local_srcs, e)
+
+        if local_srcs_changed:
+            try:
+                with open(opts.ff_local_srcs, 'w') as f:
+                    json.dump(opts.local_srcs, f)
+
+                    log('[local sources configuration]')
+                    for key, val in sorted(opts.local_srcs.items()):
+                        if not val:
+                            val = '<parent>' if key == GBL_LSRCS else '<unset>'
+                        log('({}) {}', key, val)
+
+                    success('configured root for local-sources mode')
+                    configured = True
+            except Exception as e:
+                err_flag = True
+                err('''\
+failed to write local sources file
+
+The file used to track `--local-sources` options cannot be written to.
+
+     File: {}
+    Error: {}''', opts.ff_local_srcs, e)
 
         if err_flag:
             return False
@@ -816,6 +950,30 @@ following key entry and re-try again.
                 return False
             self.opts.default_internal_pkgs = is_default_internal
 
+        if ConfKey.EXTRA_LEXCEPTS in settings:
+            d = interpret_dictionary_strings(settings[ConfKey.EXTRA_LEXCEPTS])
+            if d is None:
+                notify_invalid_value(ConfKey.EXTRA_LEXCEPTS, 'dict(str,str)')
+                return False
+
+            for key, val in d.items():
+                self.opts.spdx['exceptions'][key] = {
+                    'name': val,
+                    'deprecated': False,
+                }
+
+        if ConfKey.EXTRA_LICENSES in settings:
+            d = interpret_dictionary_strings(settings[ConfKey.EXTRA_LICENSES])
+            if d is None:
+                notify_invalid_value(ConfKey.EXTRA_LICENSES, 'dict(str,str)')
+                return False
+
+            for key, val in d.items():
+                self.opts.spdx['licenses'][key] = {
+                    'name': val,
+                    'deprecated': False,
+                }
+
         if ConfKey.LICENSE_HEADER in settings:
             license_header = interpret_string(settings[ConfKey.LICENSE_HEADER])
             if license_header is None:
