@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.api import RelengBuildOptions
 from releng_tool.defs import PackageType
 from releng_tool.engine.autotools.build import build as build_autotools
 from releng_tool.engine.cmake.build import build as build_cmake
+from releng_tool.engine.make.build import build as build_make
 from releng_tool.engine.python.build import build as build_python
+from releng_tool.engine.scons.build import build as build_scons
 from releng_tool.engine.script.build import build as build_script
 from releng_tool.util import nullish_coalescing as NC
 from releng_tool.util.api import replicate_package_attribs
@@ -14,6 +17,7 @@ from releng_tool.util.log import err
 from releng_tool.util.log import note
 import sys
 
+
 def stage(engine, pkg, script_env):
     """
     handles the build stage for a package
@@ -74,8 +78,12 @@ def stage(engine, pkg, script_env):
         builder = build_autotools
     elif pkg.type == PackageType.CMAKE:
         builder = build_cmake
+    elif pkg.type == PackageType.MAKE:
+        builder = build_make
     elif pkg.type == PackageType.PYTHON:
         builder = build_python
+    elif pkg.type == PackageType.SCONS:
+        builder = build_scons
     elif pkg.type == PackageType.SCRIPT:
         builder = build_script
 
