@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.defs import PackageInstallType
 from releng_tool.tool.cmake import CMAKE
@@ -8,6 +9,7 @@ from releng_tool.util.io import interim_working_dir
 from releng_tool.util.io import prepare_arguments
 from releng_tool.util.io import prepare_definitions
 from releng_tool.util.log import err
+from releng_tool.util.log import verbose
 from releng_tool.util.string import expand
 import os
 import posixpath
@@ -15,6 +17,17 @@ import posixpath
 #: default lib container directory
 DEFAULT_LIB_DIR = 'lib'
 
+# cmake system include directories for various language types
+CMAKE_INCLUDE_INJECT_OPTIONS = [
+    'CMAKE_CUDA_STANDARD_INCLUDE_DIRECTORIES',
+    'CMAKE_CXX_STANDARD_INCLUDE_DIRECTORIES',
+    'CMAKE_C_STANDARD_INCLUDE_DIRECTORIES',
+    'CMAKE_HIP_STANDARD_INCLUDE_DIRECTORIES',
+    'CMAKE_OBJCXX_STANDARD_INCLUDE_DIRECTORIES',
+    'CMAKE_OBJC_STANDARD_INCLUDE_DIRECTORIES',
+]
+
+
 def configure(opts):
     """
     support configuration for cmake projects
@@ -36,11 +49,19 @@ def configure(opts):
     prefix = opts.prefix
 
     base_locs = []
-    if opts._install_type == PackageInstallType.HOST:
+    if opts.install_type == PackageInstallType.HOST:
         base_locs.append(opts.host_dir)
     else:
         base_locs.append(opts.staging_dir)
-        base_locs.append(opts.target_dir)
+
+        # only reference/pre-create the target directory if this package is
+        # aimed to use the target directory
+        target_area_types = [
+            PackageInstallType.STAGING_AND_TARGET,
+            PackageInstallType.TARGET,
+        ]
+        if opts.install_type in target_area_types:
+            base_locs.append(opts.target_dir)
 
     include_locs = []
     library_locs = []
@@ -60,11 +81,12 @@ def configure(opts):
     posix_lib_dir = lib_dir.replace(os.sep, posixpath.sep)
 
     # definitions
+    compiled_include_locs = ';'.join(include_locs)
     cmake_defs = {
         # configure as RelWithDebInfo (when using multi-configuration projects)
         'CMAKE_BUILD_TYPE': 'RelWithDebInfo',
         # common paths for releng-tool sysroots
-        'CMAKE_INCLUDE_PATH': ';'.join(include_locs),
+        'CMAKE_INCLUDE_PATH': compiled_include_locs,
         'CMAKE_INSTALL_PREFIX': posix_prefix,
         'CMAKE_LIBRARY_PATH': ';'.join(library_locs),
         'CMAKE_PREFIX_PATH': ';'.join(prefix_locs),
@@ -75,6 +97,13 @@ def configure(opts):
         # `lib64`.
         'CMAKE_INSTALL_LIBDIR': posix_lib_dir,
     }
+
+    if 'releng.cmake.disable_direct_includes' not in opts._quirks:
+        for option in CMAKE_INCLUDE_INJECT_OPTIONS:
+            cmake_defs[option] = compiled_include_locs
+    else:
+        verbose('cmake direct includes disabled by quirk')
+
     if opts.conf_defs:
         cmake_defs.update(expand(opts.conf_defs))
 
@@ -86,7 +115,6 @@ def configure(opts):
 
     # argument building
     cmake_args = [
-        '--no-warn-unused-cli', # suppress common options if not used in project
     ]
     cmake_args.extend(prepare_definitions(cmake_defs, '-D'))
     cmake_args.extend(prepare_arguments(cmake_opts))
