@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.tool.cmake import CMAKE
 from releng_tool.util.io import prepare_arguments
 from releng_tool.util.io import prepare_definitions
 from releng_tool.util.log import err
-from releng_tool.util.log import verbose
 from releng_tool.util.string import expand
 
+
 def install(opts):
     """
     support installation cmake projects
@@ -26,11 +27,6 @@ def install(opts):
         err('unable to install package; cmake is not installed')
         return False
 
-    # check if the no-install flag is set
-    if opts._cmake_noinstall:
-        verbose('configured to skip install stage for cmake')
-        return True
-
     # default definitions
     cmake_defs = {
     }
@@ -55,11 +51,17 @@ def install(opts):
     cmake_args.extend(prepare_definitions(cmake_defs, '-D'))
     cmake_args.extend(prepare_arguments(cmake_opts))
 
-    # prepare environment for installation request; an environment dictionary is
-    # always needed to apply a custom DESTDIR during each install request
-    env = expand(opts.install_env)
-    if not env:
-        env = {}
+    # default environment
+    env = {
+        # always force an install to avoid possible issues with file time
+        # installation checks; releng-tool already wraps the installation
+        # event with its own control flags, so forcing an install should
+        # not cause any major issues
+        'CMAKE_INSTALL_ALWAYS': '1',
+    }
+
+    if opts.install_env:
+        env.update(expand(opts.install_env))
 
     # install to each destination
     for dest_dir in opts.dest_dirs:
