@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.api import RelengConfigureOptions
 from releng_tool.defs import PackageInstallType
 from releng_tool.defs import PackageType
 from releng_tool.engine.autotools.configure import configure as conf_autotools
 from releng_tool.engine.cmake.configure import configure as conf_cmake
+from releng_tool.engine.make.configure import configure as conf_make
+from releng_tool.engine.scons.configure import configure as conf_scons
 from releng_tool.engine.script.configure import configure as conf_script
 from releng_tool.util import nullish_coalescing as NC
 from releng_tool.util.api import replicate_package_attribs
@@ -14,6 +17,7 @@ from releng_tool.util.log import err
 from releng_tool.util.log import note
 import sys
 
+
 def stage(engine, pkg, script_env):
     """
     handles the configuration stage for a package
@@ -82,6 +86,10 @@ def stage(engine, pkg, script_env):
         configurer = conf_autotools
     elif pkg.type == PackageType.CMAKE:
         configurer = conf_cmake
+    elif pkg.type == PackageType.MAKE:
+        configurer = conf_make
+    elif pkg.type == PackageType.SCONS:
+        configurer = conf_scons
     elif pkg.type == PackageType.SCRIPT:
         configurer = conf_script
 
