@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.api import RelengExtractOptions
 from releng_tool.defs import VcsType
@@ -21,6 +22,7 @@ import os
 import shutil
 import sys
 
+
 def stage(engine, pkg):
     """
     handles the extraction stage for a package
@@ -36,8 +38,12 @@ def stage(engine, pkg):
         ``True`` if the extraction stage is completed; ``False`` otherwise
     """
 
-    # internal packages in local sources mode do not have an extraction stage
-    if pkg.is_internal and engine.opts.local_srcs:
+    # none/local-vcs-type packages do not need to fetch
+    if pkg.vcs_type in (VcsType.LOCAL, VcsType.NONE):
+        return True
+
+    # packages flagged for local sources do not have an extraction stage
+    if pkg.local_srcs:
         return True
 
     # skip packages flagged not to extract
@@ -105,7 +111,12 @@ def stage(engine, pkg):
             if not extracted:
                 return False
 
-            result = verify_hashes(pkg.hash_file, work_dir, hash_exclude)
+            # relax hash check if a both operating in a development mode and
+            # this package has development mode sources
+            hash_relaxed = engine.opts.devmode and pkg.devmode
+
+            result = verify_hashes(pkg.hash_file, work_dir,
+                exclude=hash_exclude, relaxed=hash_relaxed)
             if result == HashResult.VERIFIED:
                 pass
             elif result == HashResult.BAD_PATH:
@@ -114,8 +125,10 @@ def stage(engine, pkg):
             elif result == HashResult.EMPTY:
                 if not pkg.is_internal:
                     verbose('hash file for package is empty: ' + pkg.name)
-            elif result in (HashResult.BAD_FORMAT, HashResult.MISMATCH,
-                    HashResult.MISSING_LISTED, HashResult.UNSUPPORTED):
+            elif result in (HashResult.MISMATCH, HashResult.MISSING_LISTED):
+                if not hash_relaxed:
+                    return False
+            elif result in (HashResult.BAD_FORMAT, HashResult.UNSUPPORTED):
                 return False
             else:
                 err('invalid extract operation (internal error; '
