@@ -1,20 +1,25 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.api import RelengInstallOptions
 from releng_tool.defs import PackageInstallType
 from releng_tool.defs import PackageType
 from releng_tool.engine.autotools.install import install as install_autotools
 from releng_tool.engine.cmake.install import install as install_cmake
+from releng_tool.engine.make.install import install as install_make
 from releng_tool.engine.python.install import install as install_python
+from releng_tool.engine.scons.install import install as install_scons
 from releng_tool.engine.script.install import install as install_script
 from releng_tool.util import nullish_coalescing as NC
 from releng_tool.util.api import replicate_package_attribs
 from releng_tool.util.io import interim_working_dir
 from releng_tool.util.log import err
 from releng_tool.util.log import note
+from releng_tool.util.log import verbose
 import sys
 
+
 def stage(engine, pkg, script_env):
     """
     handles the installation stage for a package
@@ -31,6 +36,20 @@ def stage(engine, pkg, script_env):
         ``True`` if the installation stage is completed; ``False`` otherwise
     """
 
+    # check if a package is not using the installation stage
+    skip_install = False
+
+    if pkg.type == PackageType.CMAKE:
+        skip_install = pkg.cmake_noinstall
+    elif pkg.type == PackageType.MAKE:
+        skip_install = pkg.make_noinstall
+    elif pkg.type == PackageType.SCONS:
+        skip_install = pkg.scons_noinstall
+
+    if skip_install:
+        verbose('skipping installation stage for {}', pkg.name)
+        return True
+
     note('installing {}...', pkg.name)
     sys.stdout.flush()
 
@@ -86,8 +105,12 @@ def stage(engine, pkg, script_env):
         installer = install_autotools
     elif pkg.type == PackageType.CMAKE:
         installer = install_cmake
+    elif pkg.type == PackageType.MAKE:
+        installer = install_make
     elif pkg.type == PackageType.PYTHON:
         installer = install_python
+    elif pkg.type == PackageType.SCONS:
+        installer = install_scons
     elif pkg.type == PackageType.SCRIPT:
         installer = install_script
 
