@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from glob import glob
+from releng_tool.defs import VcsType
 from releng_tool.tool.patch import PATCH
 from releng_tool.util.log import err
 from releng_tool.util.log import note
@@ -13,7 +15,8 @@ import sys
 #: filename of the script to execute custom patching operations (if any)
 PATCH_SCRIPT = 'patch'
 
-def stage(engine, pkg, script_env):
+
+def stage(engine, pkg, script_env):  # noqa: ARG001
     """
     handles the patching stage for a package
 
@@ -29,15 +32,18 @@ def stage(engine, pkg, script_env):
         ``True`` if the patching stage is completed; ``False`` otherwise
     """
 
-    if pkg.is_internal:
-        # internal packages in local sources mode do not have a patch stage
-        if engine.opts.local_srcs:
-            return True
+    # local-vcs-type packages do not need to patch
+    if pkg.vcs_type is VcsType.LOCAL:
+        return True
 
-        # internal packages in development mode that specify a development
-        # revision will not perform the patch stage
-        if engine.opts.devmode and pkg.has_devmode_option:
-            return True
+    # internal packages flagged for local sources do not have a patch stage
+    if pkg.is_internal and pkg.local_srcs:
+        return True
+
+    # packages which have a custom revision while in development mode will
+    # not perform the patch stage
+    if pkg.devmode:
+        return True
 
     note('patching {}...', pkg.name)
     sys.stdout.flush()
