@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 def releng_setup(app):
     seed_ext_type = RelengSeedExtension
@@ -7,12 +8,15 @@ def releng_setup(app):
     app.add_extract_type('ext-seed', seed_ext_type)
     app.add_package_type('ext-seed', seed_ext_type)
 
+
 class RelengSeedExtension:
     def fetch(self, name, opts):
         print('(seed) package', opts.name, 'has been fetched')
 
         try:
-            open(opts.cache_file, 'w').close()
+            with open(opts.cache_file, 'w'):
+                pass
+
             return opts.cache_file
         except IOError:
             return None
