@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.tool.tar import TAR
 from releng_tool.util.io import ensure_dir_exists
@@ -21,6 +22,7 @@ TAR_SUPPORTED = (
     'tgz',
 )
 
+
 def extract(opts):
     """
     support extraction of an archive into a build directory
@@ -103,6 +105,15 @@ def extract(opts):
                 try:
                     def tar_extract(members, strip_count):
                         for member in members:
+                            # do not process a tar that has a member which
+                            # extracts outside of the work directory
+                            target_path = os.path.join(work_dir, member.name)
+                            target_path = os.path.abspath(target_path)
+                            common_prefix = os.path.commonprefix(
+                                [work_dir, target_path])
+                            if common_prefix != work_dir:
+                                raise Exception('path traversal detected')
+
                             # strip members from package defined count
                             if strip_count > 0:
                                 np = os.path.normpath(member.name)
