@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from __future__ import print_function
 from releng_tool.util.log import err
@@ -18,6 +19,7 @@ except ImportError:
 #: size of blocks read when downloading a resource
 REQUEST_READ_BLOCKSIZE = 8192
 
+
 def fetch(opts):
     """
     support fetching from url sources
@@ -87,6 +89,7 @@ def fetch(opts):
     log('completed download ({})', display_size(read))
     return cache_file
 
+
 def display_size(val):
     """
     return a human-readable count value for the provided byte count
@@ -100,9 +103,11 @@ def display_size(val):
     Returns:
         the human-readable size
     """
+
+    SZ = 1024.
     for unit in ['B', 'KiB', 'MiB', 'GiB']:
-        if abs(val) < 1024.:
+        if abs(val) < SZ:
             return '{:3.1f} {}'.format(val, unit)
-        val /= 1024.
+        val /= SZ
 
     return '{:.1f} TiB'.format(val)
