@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from releng_tool.util.enum import Enum
 import hashlib
 
+
 class PkgKeyType(Enum):
     """
     package key type
@@ -15,6 +17,7 @@ class PkgKeyType(Enum):
         BOOL: boolean value
         DICT: dictionary value
         DICT_STR_STR: dictionary of string pairs value
+        DICT_STR_STR_OR_STR: dictionary of string pairs or a string value
         DICT_STR_STR_OR_STRS: dictionary of string pairs or strings value
         STR: single string value
         STRS: one or more strings value
@@ -24,12 +27,14 @@ class PkgKeyType(Enum):
     BOOL = 'bool'
     DICT = 'dict'
     DICT_STR_STR = 'dict_str_str'
+    DICT_STR_STR_OR_STR = 'dict_str_str_or_str'
     DICT_STR_STR_OR_STRS = 'dict_str_str_or_strs'
     STR = 'str'
     STRS = 'strs'
     INT_NONNEGATIVE = 'int_nonnegative'
     INT_POSITIVE = 'int_positive'
 
+
 def pkg_cache_key(site):
     """
     generate a cache key for a provided package's site
@@ -40,7 +45,8 @@ def pkg_cache_key(site):
     Returns:
         the cache key
     """
-    return hashlib.sha1(site.encode('utf_8')).hexdigest()
+    return hashlib.sha1(site.encode('utf_8')).hexdigest()  # noqa: S324
+
 
 def pkg_key(pkg, type_):
     """
