@@ -1,8 +1,12 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
+from collections import OrderedDict
+from releng_tool.defs import GBL_LSRCS
 from releng_tool.defs import PackageInstallType
 from releng_tool.defs import PackageType
+from releng_tool.defs import PythonSetupType
 from releng_tool.defs import Rpk
 from releng_tool.defs import VcsType
 from releng_tool.packages import PkgKeyType
@@ -19,6 +23,7 @@ from releng_tool.packages.exceptions import RelengToolMissingPackageSite
 from releng_tool.packages.exceptions import RelengToolUnknownExtractType
 from releng_tool.packages.exceptions import RelengToolUnknownInstallType
 from releng_tool.packages.exceptions import RelengToolUnknownPackageType
+from releng_tool.packages.exceptions import RelengToolUnknownPythonSetupType
 from releng_tool.packages.exceptions import RelengToolUnknownVcsType
 from releng_tool.packages.package import RelengPackage
 from releng_tool.opts import RELENG_CONF_EXTENDED_NAME
@@ -31,27 +36,34 @@ from releng_tool.util.log import debug
 from releng_tool.util.log import verbose
 from releng_tool.util.log import warn
 from releng_tool.util.sort import TopologicalSorter
+from releng_tool.util.spdx import spdx_extract
 from releng_tool.util.string import expand
 from releng_tool.util.string import interpret_dictionary_strings
 from releng_tool.util.string import interpret_string
 from releng_tool.util.string import interpret_strings
 from releng_tool.util.string import interpret_zero_to_one_strings
 import pickle
+import posixpath
 import pprint
 import os
 import traceback
+import sys
 
 try:
     from urllib.parse import urlparse
 except ImportError:
     from urlparse import urlparse
 
+# special value for a "default" entry in a dictionary
+DEFAULT_ENTRY = '*'
+
 #: default strip-count value for packages
 DEFAULT_STRIP_COUNT = 1
 
 # cache name for dvcs database
 DVCS_CACHE_FNAME = '.dvcsdb'
 
+
 class RelengPackageManager:
     """
     a releng package manager
@@ -109,6 +121,7 @@ class RelengPackageManager:
         self._register_conf(Rpk.GIT_REFSPECS, PkgKeyType.STRS)
         self._register_conf(Rpk.GIT_SUBMODULES, PkgKeyType.BOOL)
         self._register_conf(Rpk.GIT_VERIFY_REVISION, PkgKeyType.BOOL)
+        self._register_conf(Rpk.HOST_PROVIDES, PkgKeyType.STRS)
         self._register_conf(Rpk.INSTALL_DEFS, PkgKeyType.DICT_STR_STR)
         self._register_conf(Rpk.INSTALL_ENV, PkgKeyType.DICT_STR_STR)
         self._register_conf(Rpk.INSTALL_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
@@ -116,16 +129,19 @@ class RelengPackageManager:
         self._register_conf(Rpk.INTERNAL, PkgKeyType.BOOL)
         self._register_conf(Rpk.LICENSE, PkgKeyType.STRS)
         self._register_conf(Rpk.LICENSE_FILES, PkgKeyType.STRS)
+        self._register_conf(Rpk.MAKE_NOINSTALL, PkgKeyType.BOOL)
         self._register_conf(Rpk.NO_EXTRACTION, PkgKeyType.BOOL)
         self._register_conf(Rpk.PREFIX, PkgKeyType.STR)
         self._register_conf(Rpk.PYTHON_INTERPRETER, PkgKeyType.STR)
-        self._register_conf(Rpk.REVISION, PkgKeyType.STR)
-        self._register_conf(Rpk.SITE, PkgKeyType.STR)
+        self._register_conf(Rpk.PYTHON_SETUP_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.REVISION, PkgKeyType.DICT_STR_STR_OR_STR)
+        self._register_conf(Rpk.SCONS_NOINSTALL, PkgKeyType.BOOL)
+        self._register_conf(Rpk.SITE, PkgKeyType.DICT_STR_STR_OR_STR)
         self._register_conf(Rpk.SKIP_REMOTE_CONFIG, PkgKeyType.BOOL)
         self._register_conf(Rpk.SKIP_REMOTE_SCRIPTS, PkgKeyType.BOOL)
         self._register_conf(Rpk.STRIP_COUNT, PkgKeyType.INT_NONNEGATIVE)
         self._register_conf(Rpk.TYPE, PkgKeyType.STR)
-        self._register_conf(Rpk.VCS_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.VCS_TYPE, PkgKeyType.DICT_STR_STR_OR_STR)
         self._register_conf(Rpk.VERSION, PkgKeyType.STR)
 
         # sanity check that check option is properly registered
@@ -157,7 +173,7 @@ class RelengPackageManager:
             RelengToolInvalidPackageConfiguration: when an error has been
                                                     detected loading the package
         """
-        pkgs = {}
+        pkgs = OrderedDict()
         final_deps = {}
 
         # cycle through all pending packages until the complete list is known
@@ -258,6 +274,11 @@ class RelengPackageManager:
                 'script': script,
             })
 
+        pkg_def_dir = os.path.abspath(os.path.join(script, os.pardir))
+        self.script_env['DEFAULT_REVISION'] = DEFAULT_ENTRY
+        self.script_env['DEFAULT_SITE'] = DEFAULT_ENTRY
+        self.script_env['PKG_DEFDIR'] = pkg_def_dir
+
         try:
             env = run_script(script, self.script_env, catch=False)
         except Exception as e:
@@ -276,53 +297,105 @@ class RelengPackageManager:
 
         # version/revision extraction first
         #
-        # Attempt to check the version first since it will be the most commonly
-        # used package field -- rather initially fail on a simple field first
-        # (for new packages and/or developers) than breaking on a possibly more
-        # complex field below. Note that the version field is optional, in cases
-        # where a package type does not need a version entry (e.g. sites which
-        # do not require a version value for fetching or there is not revision
-        # value to use instead).
+        # Pull the version/revision(s) information first. This information can
+        # be used for pulling specific revisions for sources and other minor
+        # things such where projects may be extracted to, etc. The "version"
+        # value is typically used for naming output folders, display, etc.;
+        # where as a "revision" value is typically for identify what sources
+        # to clone from (for DVCS). If a revision value is not specified, it
+        # will be populated with the same value as the version value.
         #
-        # Note that when in development mode, the development-mode revision
-        # (if any is set) needs to be checked as well. This value may override
-        # the package's version value.
+        # A package may have multiple revisions. Typically a "default" (`*`)
+        # revision or other revisions which may be triggered based of a user
+        # configuring for "development mode". At the end of parsing, the
+        # `pkg_revision` value should be a single string value (even empty).
+        #
+        # Support still exists for a development revision (`DEVMODE_REVISION`)
+        # value (although deprecated). The package's revision may be overriden
+        # with this value based on various scenarios.
 
         # version
-        pkg_version = self._fetch(Rpk.VERSION)
-
-        if not pkg_version:
-            pkg_version = ''
+        pkg_version = self._fetch(Rpk.VERSION, default='')
 
         pkg_version_key = pkg_key(name, Rpk.VERSION)
         expand_extra[pkg_version_key] = pkg_version
 
-        # development mode revision
-        pkg_has_devmode_option = False
+        # development mode revision (may deprecate)
         pkg_devmode_revision = self._fetch(Rpk.DEVMODE_REVISION,
             allow_expand=True, expand_extra=expand_extra)
 
-        if pkg_devmode_revision:
-            pkg_has_devmode_option = True
+        # revisions
+        pkg_devmode = False
+        pkg_revision = None
 
-            if opts.revision_override and name in opts.revision_override:
-                pkg_devmode_revision = opts.revision_override[name]
+        if opts.revision_override:
+            pkg_revision = opts.revision_override.get(name)
+            pkg_devmode = bool(pkg_revision)
 
-            if opts.devmode:
-                pkg_version = pkg_devmode_revision
-                expand_extra[pkg_version_key] = pkg_version
-
-        # revision
-        if opts.revision_override and name in opts.revision_override:
-            pkg_revision = opts.revision_override[name]
-        else:
-            pkg_revision = self._fetch(Rpk.REVISION,
+        if not pkg_revision:
+            pkg_revision_raw = self._fetch(Rpk.REVISION,
                 allow_expand=True, expand_extra=expand_extra)
-        if opts.devmode and pkg_devmode_revision:
-            pkg_revision = pkg_devmode_revision
-        elif not pkg_revision:
+            if pkg_revision_raw:
+                # user has a defined a series of revision entries -- find the
+                # revision value based off out current mode (i.e. if in
+                # development mode, use the approriate key; otherwise default
+                # to a `*` key, if it exists)
+                if isinstance(pkg_revision_raw, dict):
+                    pkg_revision = pkg_revision_raw.get(opts.devmode)
+                    pkg_devmode = bool(pkg_revision)
+
+                    # no explicit revision, check the "default/any" revision
+                    if not pkg_revision:
+                        pkg_revision = pkg_revision_raw.get(DEFAULT_ENTRY)
+
+                    # no default revision to use for this mode, check if the
+                    # deprecated devmode-revision value is set
+                    if not pkg_revision and \
+                            opts.devmode and pkg_devmode_revision:
+                        pkg_revision = pkg_devmode_revision
+                        pkg_devmode = bool(pkg_revision)
+
+                    # lastly, if no revision has been found, default to the
+                    # package's version
+                    if not pkg_revision:
+                        pkg_revision = pkg_version
+
+                # if this is a single revision string, use this value for
+                # the revision; with the exception if we are in development
+                # mode and the deprecated devmode-revision value is set
+                else:
+                    if opts.devmode and pkg_devmode_revision:
+                        pkg_revision = pkg_devmode_revision
+                        pkg_devmode = True
+                    else:
+                        pkg_revision = pkg_revision_raw
+
+            # if we do not have package revision information provided, we
+            # will default to the using the package's version; with the
+            # exception if we are in development mode and the deprecated
+            # devmode-revision value is set
+            else:
+                if opts.devmode and pkg_devmode_revision:
+                    pkg_revision = pkg_devmode_revision
+                    pkg_devmode = True
+                else:
+                    pkg_revision = pkg_version
+
+        # always ensure a revision is set; use the version value if no
+        # explicit revision is provided
+        if not pkg_revision:
             pkg_revision = pkg_version
 
+        pkg_revision_key = pkg_key(name, Rpk.REVISION)
+        expand_extra[pkg_revision_key] = pkg_revision
+
+        # if we have replaced the revision with a development-specific value,
+        # also replace the package's version to better reflect the version of
+        # the package is not expected
+        if pkg_devmode:
+            pkg_version = pkg_revision
+            expand_extra[pkg_version_key] = pkg_version
+
         # site / vcs-site detection
         #
         # After extracted required version information, the site / VCS type
@@ -338,23 +411,62 @@ class RelengPackageManager:
             # source when experiencing network connectivity issues.
             pkg_site = opts.sites_override[name]
         else:
-            pkg_site = self._fetch(Rpk.SITE,
+            pkg_site = None
+            pkg_site_raw = self._fetch(Rpk.SITE,
                 allow_expand=True, expand_extra=expand_extra)
 
+            if pkg_site_raw:
+                # user has a defined a series of site entries -- find the
+                # site value based off out current mode (i.e. if in
+                # development mode, use the approriate key; otherwise default
+                # to a `*` key, if it exists)
+                if isinstance(pkg_site_raw, dict):
+                    pkg_site = pkg_site_raw.get(opts.devmode)
+
+                    # no explicit site, check the "default/any" site
+                    if not pkg_site:
+                        pkg_site = pkg_site_raw.get(DEFAULT_ENTRY)
+                else:
+                    pkg_site = pkg_site_raw
+
+        # On Windows, if a file site is provided, ensure the path value is
+        # converted to a posix-styled path, to prevent issues with `urlopen`
+        # being provided an unescaped path string
+        if sys.platform == 'win32' and \
+                pkg_site and pkg_site.startswith('file://'):
+            pkg_site = pkg_site[len('file://'):]
+            abs_site = os.path.isabs(pkg_site)
+            pkg_site = pkg_site.replace(os.sep, posixpath.sep)
+            if abs_site:
+                pkg_site = '/' + pkg_site
+            pkg_site = 'file://' + pkg_site
+
         # vcs-type
         pkg_vcs_type = None
         pkg_vcs_type_raw = self._fetch(Rpk.VCS_TYPE)
         if pkg_vcs_type_raw:
-            pkg_vcs_type_raw = pkg_vcs_type_raw.lower()
-            if pkg_vcs_type_raw in VcsType:
-                pkg_vcs_type = pkg_vcs_type_raw
-            elif pkg_vcs_type_raw in self.registry.fetch_types:
-                pkg_vcs_type = pkg_vcs_type_raw
-            else:
-                raise RelengToolUnknownVcsType({
-                    'pkg_name': name,
-                    'pkg_key': pkg_key(name, Rpk.VCS_TYPE),
-                })
+            # user has a defined a series of vcs-type entries -- find the
+            # vcs-type value based off out current mode (i.e. if in
+            # development mode, use the approriate key; otherwise default
+            # to a `*` key, if it exists)
+            if isinstance(pkg_vcs_type_raw, dict):
+                if pkg_vcs_type_raw.get(opts.devmode):
+                    pkg_vcs_type_raw = pkg_vcs_type_raw.get(opts.devmode)
+                # no explicit site, check the "default/any" site
+                else:
+                    pkg_vcs_type_raw = pkg_vcs_type_raw.get(DEFAULT_ENTRY)
+
+            if pkg_vcs_type_raw:
+                pkg_vcs_type_raw = pkg_vcs_type_raw.lower()
+                if pkg_vcs_type_raw in VcsType:
+                    pkg_vcs_type = pkg_vcs_type_raw
+                elif pkg_vcs_type_raw in self.registry.fetch_types:
+                    pkg_vcs_type = pkg_vcs_type_raw
+                else:
+                    raise RelengToolUnknownVcsType({
+                        'pkg_name': name,
+                        'pkg_key': pkg_key(name, Rpk.VCS_TYPE),
+                    })
 
         if not pkg_vcs_type:
             if pkg_site:
@@ -443,6 +555,9 @@ class RelengPackageManager:
                     'pkg_key': pkg_key(name, Rpk.EXTRACT_TYPE),
                 })
 
+        # host tools provided
+        pkg_host_provides = self._fetch(Rpk.HOST_PROVIDES)
+
         # is-external
         pkg_is_external = self._fetch(Rpk.EXTERNAL)
 
@@ -590,12 +705,42 @@ class RelengPackageManager:
             pkg_nv = name
 
         pkg_build_output_dir = os.path.join(opts.build_dir, pkg_nv)
-        pkg_def_dir = os.path.abspath(os.path.join(script, os.pardir))
-        if pkg_vcs_type == VcsType.LOCAL:
-            pkg_build_dir = pkg_def_dir
-        elif opts.local_srcs and pkg_is_internal:
-            container_dir = os.path.dirname(opts.root_dir)
-            pkg_build_dir = os.path.join(container_dir, name)
+
+        pkg_is_local = pkg_vcs_type == VcsType.LOCAL
+        if pkg_is_local:
+            new_local_dir = os.path.join(pkg_def_dir, 'local')
+            if os.path.isdir(new_local_dir):
+                pkg_build_dir = new_local_dir
+            else:
+                # deprecated local directory working inside definition folder
+                warn('local-defined package missing local folder: {}', name)
+                pkg_build_dir = pkg_def_dir
+        else:
+            pkg_build_dir = pkg_build_output_dir
+
+        # check if an internal package is configured to point to a local
+        # directory for sources
+        pkg_local_srcs = False
+        if pkg_is_internal and not pkg_is_local and opts.local_srcs:
+            # specific package name reference in the local sources; either is
+            # set to the path to use, or is set to `None` to indicate at this
+            # package should not be retrieved locally
+            if name in opts.local_srcs:
+                if opts.local_srcs[name]:
+                    pkg_build_dir = opts.local_srcs[name]
+                    pkg_local_srcs = True
+
+            # check if the "global" local sources path exists; either set to
+            # a specific path, or set to `None` to indicate that it will use
+            # the parent path based off the root directory
+            elif GBL_LSRCS in opts.local_srcs:
+                if opts.local_srcs[GBL_LSRCS]:
+                    container_dir = opts.local_srcs[GBL_LSRCS]
+                else:
+                    container_dir = os.path.dirname(opts.root_dir)
+
+                pkg_build_dir = os.path.join(container_dir, name)
+                pkg_local_srcs = True
 
             if pkg_build_dir == opts.root_dir:
                 raise RelengToolConflictingLocalSrcsPath({
@@ -603,14 +748,15 @@ class RelengPackageManager:
                     'root': opts.root_dir,
                     'path': pkg_build_dir,
                 })
-        else:
-            pkg_build_dir = pkg_build_output_dir
+
         if pkg_build_subdir:
             pkg_build_subdir = os.path.join(pkg_build_dir, pkg_build_subdir)
+
+        cache_dir = os.path.join(opts.dl_dir, name)
         if cache_ext:
-            pkg_cache_file = os.path.join(opts.dl_dir, pkg_nv + '.' + cache_ext)
+            pkg_cache_file = os.path.join(cache_dir, pkg_nv + '.' + cache_ext)
         else:
-            pkg_cache_file = os.path.join(opts.dl_dir, pkg_nv)
+            pkg_cache_file = os.path.join(cache_dir, pkg_nv)
 
         # Select sources (like CMake-based projects) may wish to be using
         # out-of-source tree builds. For supported project types, adjust the
@@ -688,6 +834,7 @@ class RelengPackageManager:
         pkg.cache_dir = pkg_cache_dir
         pkg.cache_file = pkg_cache_file
         pkg.def_dir = pkg_def_dir
+        pkg.devmode = pkg_devmode
         pkg.devmode_ignore_cache = pkg_devmode_ignore_cache
         pkg.extract_type = pkg_extract_type
         pkg.git_config = pkg_git_config
@@ -695,9 +842,10 @@ class RelengPackageManager:
         pkg.git_refspecs = pkg_git_refspecs
         pkg.git_submodules = pkg_git_submodules
         pkg.git_verify_revision = pkg_git_verify_revision
-        pkg.has_devmode_option = pkg_has_devmode_option
         pkg.hash_file = os.path.join(pkg_def_dir, name + '.hash')
+        pkg.host_provides = pkg_host_provides
         pkg.is_internal = pkg_is_internal
+        pkg.local_srcs = pkg_local_srcs
         pkg.no_extraction = pkg_no_extraction
         pkg.revision = pkg_revision
         pkg.site = pkg_site
@@ -710,12 +858,12 @@ class RelengPackageManager:
         self._apply_postinit_options(pkg)
 
         # (additional environment helpers)
-        for env in (os.environ, env):
-            env[pkg_key(name, 'BUILD_DIR')] = pkg_build_dir
-            env[pkg_key(name, 'BUILD_OUTPUT_DIR')] = pkg_build_output_dir
-            env[pkg_key(name, 'DEFDIR')] = pkg_def_dir
-            env[pkg_key(name, 'NAME')] = name
-            env[pkg_key(name, 'REVISION')] = pkg_revision
+        for ctx in (os.environ, env):
+            ctx[pkg_key(name, 'BUILD_DIR')] = pkg_build_dir
+            ctx[pkg_key(name, 'BUILD_OUTPUT_DIR')] = pkg_build_output_dir
+            ctx[pkg_key(name, 'DEFDIR')] = pkg_def_dir
+            ctx[pkg_key(name, 'NAME')] = name
+            ctx[pkg_key(name, 'REVISION')] = pkg_revision
         os.environ[pkg_key(name, Rpk.VERSION)] = pkg_version
 
         # (internals)
@@ -765,6 +913,8 @@ class RelengPackageManager:
                                                     option
         """
 
+        opts = self.opts
+
         # ######################################################################
         # (common)
         # ######################################################################
@@ -798,6 +948,34 @@ class RelengPackageManager:
         if pkg.license is None:
             pkg.license = self._fetch(Rpk.LICENSE)
 
+        if pkg.license and 'releng.disable_spdx_check' not in opts.quirks:
+            for license_entry in pkg.license:
+                parsed, licenses, exceptions = spdx_extract(license_entry)
+
+                for nested_license in licenses:
+                    entry = opts.spdx['licenses'].get(nested_license, None)
+                    if entry:
+                        if entry['deprecated']:
+                            warn('deprecated spdx license detected ({}): {}',
+                                nested_license, pkg.name)
+                    else:
+                        warn('unknown spdx license detected ({}): {}',
+                            nested_license, pkg.name)
+
+                for exception in exceptions:
+                    entry = opts.spdx['exceptions'].get(exception, None)
+                    if entry:
+                        if entry['deprecated']:
+                            warn('deprecated spdx license exception detected '
+                                 '({}): {}', exception, pkg.name)
+                    else:
+                        warn('unknown spdx license exception detected ({}): {}',
+                            exception, pkg.name)
+
+                if not parsed:
+                    warn('unexpected spdx license format detected: {}',
+                        pkg.name)
+
         # license files
         if pkg.license_files is None:
             pkg.license_files = self._fetch(Rpk.LICENSE_FILES)
@@ -862,14 +1040,50 @@ class RelengPackageManager:
         if pkg.cmake_noinstall is None:
             pkg.cmake_noinstall = self._fetch(Rpk.CMAKE_NOINSTALL)
 
+        # ######################################################################
+        # (package type - make)
+        # ######################################################################
+
+        # make noinstall flag
+        if pkg.make_noinstall is None:
+            pkg.make_noinstall = self._fetch(Rpk.MAKE_NOINSTALL)
+
         # ######################################################################
         # (package type - python)
         # ######################################################################
 
         # python interpreter
         if pkg.python_interpreter is None:
-            pkg_python_interpreter = self._fetch(Rpk.PYTHON_INTERPRETER)
-            pkg.python_interpreter = pkg_python_interpreter
+            pkg.python_interpreter = self._fetch(Rpk.PYTHON_INTERPRETER)
+
+        # python setup type
+        if pkg.python_setup_type is None:
+            python_setup_type_raw = self._fetch(Rpk.PYTHON_SETUP_TYPE)
+            if python_setup_type_raw:
+                python_setup_type_raw = python_setup_type_raw.lower()
+                if python_setup_type_raw in PythonSetupType:
+                    pkg.python_setup_type = python_setup_type_raw
+                else:
+                    raise RelengToolUnknownPythonSetupType({
+                        'pkg_name': pkg.name,
+                        'pkg_key': pkg_key(pkg.name, Rpk.PYTHON_SETUP_TYPE),
+                    })
+
+        # ######################################################################
+        # (package type - scons)
+        # ######################################################################
+
+        # scons noinstall flag
+        if pkg.scons_noinstall is None:
+            pkg.scons_noinstall = self._fetch(Rpk.SCONS_NOINSTALL)
+
+        # ######################################################################
+        # (post checks)
+        # ######################################################################
+
+        if pkg.host_provides and pkg.install_type != PackageInstallType.HOST:
+            warn('non-host package providing host package: {}', pkg.name)
+            pkg.host_provides = None
 
     def load_remote_configuration(self, pkg):
         """
@@ -889,7 +1103,7 @@ class RelengPackageManager:
         """
 
         target_remote_configurations = [
-            os.path.join(pkg.build_dir, RELENG_CONF_EXTENDED_NAME)
+            os.path.join(pkg.build_dir, RELENG_CONF_EXTENDED_NAME),
         ]
 
         if pkg.build_subdir:
@@ -979,7 +1193,6 @@ class RelengPackageManager:
             RelengToolInvalidPackageKeyValue: value type is invalid for the key
         """
 
-        self._active_key = key
         type_ = self._key_types[key]
         value = default
 
@@ -1019,6 +1232,14 @@ class RelengPackageManager:
                     value = expand(value, expand_extra)
                 if value is None:
                     raise_kv_exception('dict(str,str)')
+            elif type_ == PkgKeyType.DICT_STR_STR_OR_STR:
+                value = interpret_string(raw_value)
+                if not value:
+                    value = raw_value
+                    if not isinstance(value, dict):
+                        raise_kv_exception('dict(str,str) or string')
+                if allow_expand:
+                    value = expand(value, expand_extra)
             elif type_ == PkgKeyType.DICT_STR_STR_OR_STRS:
                 value = interpret_zero_to_one_strings(raw_value)
                 if allow_expand:
@@ -1090,7 +1311,7 @@ class RelengPackageManager:
         try:
             with open(self._dvcs_cache_fname, 'wb') as f:
                 pickle.dump(self._dvcs_cache, f,
-                    protocol=2) # 2 for py2/py3 support
+                    protocol=2)  # 2 for py2/py3 support
             debug('saved dvcs cache')
         except IOError:
             verbose('failed to save dvcs cache')
