@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
-# Copyright 2021-2022 releng-tool
+# Copyright 2021-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from contextlib import contextmanager
 from releng_tool.defs import GlobalAction
 from releng_tool.defs import PkgAction
-from releng_tool.defs import VcsType
 from releng_tool.engine.bootstrap import stage as bootstrap_stage
 from releng_tool.engine.build import stage as build_stage
 from releng_tool.engine.configure import stage as configure_stage
@@ -22,11 +22,12 @@ from releng_tool.packages.exceptions import RelengToolInstallStageFailure
 from releng_tool.packages.exceptions import RelengToolLicenseStageFailure
 from releng_tool.packages.exceptions import RelengToolPatchStageFailure
 from releng_tool.packages.exceptions import RelengToolPostStageFailure
+from releng_tool.util.enum import Enum
 from releng_tool.util.file_flags import FileFlag
 from releng_tool.util.file_flags import check_file_flag
 from releng_tool.util.file_flags import process_file_flag
 from releng_tool.util.io import ensure_dir_exists
-from releng_tool.util.io import path_copy
+from releng_tool.util.io_copy import path_copy
 from releng_tool.util.log import debug
 from releng_tool.util.log import err
 from releng_tool.util.log import note
@@ -35,6 +36,25 @@ import os
 import subprocess
 import sys
 
+
+class PipelineResult(Enum):
+    """
+    pipeline process result
+
+    The result state from processing a pipeline stage. This enumeration is
+    primarily used to flag if a pipeline to continue processing stages
+    until completion, or if the pipeline should stop processing. Note
+    that a stopped pipeline does not indicate there was an error running
+    the pipeline.
+
+    Attributes:
+        CONTINUE: pipeline should continue processing
+        STOP: pipeline should stop processing
+    """
+    CONTINUE = 'continue'
+    STOP = 'stop'
+
+
 class RelengPackagePipeline:
     """
     a releng package pipeline
@@ -48,14 +68,11 @@ class RelengPackagePipeline:
         script_env: the script environment to load
 
     Attributes:
-        license_files: dictionary of package-to-license entities found when
-                        processing packages
         opts: options used to configure the engine
         script_env: the script environment to load
     """
     def __init__(self, engine, opts, script_env):
         self.engine = engine
-        self.license_files = {}
         self.opts = opts
         self.script_env = script_env
 
@@ -83,6 +100,7 @@ class RelengPackagePipeline:
         Returns:
             returns whether or not the pipeline should continue processing
         """
+
         gaction = self.opts.gbl_action
         paction = self.opts.pkg_action
         target = self.opts.target_action
@@ -90,19 +108,13 @@ class RelengPackagePipeline:
         # skip if generating license information and no license
         # files exist for this package
         if gaction == GlobalAction.LICENSES and not pkg.license_files:
-            return True
-
-        # prepare environment
-        pkg_env = self._stage_env(pkg)
+            return PipelineResult.CONTINUE
 
         # extracting
-        flag = pkg._ff_extract
-        if check_file_flag(flag) == FileFlag.NO_EXIST:
+        fflag = pkg._ff_extract
+        if check_file_flag(fflag) == FileFlag.NO_EXIST:
             self.engine.stats.track_duration_start(pkg.name, 'extract')
-            # none/local-vcs-type packages do not need to fetch
-            if pkg.vcs_type in (VcsType.LOCAL, VcsType.NONE):
-                pass
-            elif not extract_stage(self.engine, pkg):
+            if not extract_stage(self.engine, pkg):
                 raise RelengToolExtractionStageFailure
             # now that the extraction stage has (most likely)
             # created a build directory, ensure the output directory
@@ -110,59 +122,75 @@ class RelengPackagePipeline:
             if not ensure_dir_exists(pkg.build_output_dir):
                 raise RelengToolExtractionStageFailure
             self.engine.stats.track_duration_end(pkg.name, 'extract')
-            if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                return False
+            if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                return PipelineResult.STOP
         if gaction == GlobalAction.EXTRACT:
-            return True
+            return PipelineResult.CONTINUE
         if paction == PkgAction.EXTRACT and pkg.name == target:
-            return False
+            return PipelineResult.STOP
+
+        # process the package data with a package-specific environment
+        with self._stage_env(pkg) as pkg_env:
+            return self._process_data(pkg, pkg_env)
+
+    def _process_data(self, pkg, pkg_env):
+        """
+        process a provided package (data)
+
+        This request will process a package through the various stages (if these
+        stages are applicable to the current run state and have yet been
+        executed from a previous run). A package-specific script environment
+        will be prepared and a package will go through the process of:
+
+        - Patching
+        - License management (if needed)
+        - Bootstrapping
+        - Configuration
+        - Building
+        - Installing
+        - Post-processing
+
+        Args:
+            pkg: the package to process
+            pkg_env: the package environment
+
+        Returns:
+            returns whether or not the pipeline should continue processing
+        """
+
+        gaction = self.opts.gbl_action
+        paction = self.opts.pkg_action
+        target = self.opts.target_action
 
         # patching
-        flag = pkg._ff_patch
-        if check_file_flag(flag) == FileFlag.NO_EXIST:
+        fflag = pkg._ff_patch
+        if check_file_flag(fflag) == FileFlag.NO_EXIST:
             self.engine.stats.track_duration_start(pkg.name, 'patch')
-            # local-vcs-type packages do not need to patch
-            if pkg.vcs_type is VcsType.LOCAL:
-                pass
-            elif not patch_stage(self.engine, pkg, pkg_env):
+            if not patch_stage(self.engine, pkg, pkg_env):
                 raise RelengToolPatchStageFailure
             self.engine.stats.track_duration_end(pkg.name, 'patch')
-            if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                return False
+            if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                return PipelineResult.STOP
         if gaction == GlobalAction.PATCH:
-            return True
+            return PipelineResult.CONTINUE
         if paction == PkgAction.PATCH and pkg.name == target:
-            return False
+            return PipelineResult.STOP
 
         # handle license generation request
         #
         # If the user has requested to generate license information,
         # pull license assets from the extract package content.
         # license(s)
-        flag = pkg._ff_license
-        if check_file_flag(flag) == FileFlag.NO_EXIST:
+        fflag = pkg._ff_license
+        if check_file_flag(fflag) == FileFlag.NO_EXIST:
             if not self._stage_license(pkg):
                 raise RelengToolLicenseStageFailure
-            if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                return False
-
-        if pkg.license_files:
-            version_desc = pkg.version
-            if not version_desc and pkg.revision:
-                version_desc = pkg.revision
-
-            self.license_files[pkg.name] = {
-                'files': [],
-                'version': version_desc,
-            }
-            for file in pkg.license_files:
-                file = os.path.join(pkg.build_dir, file)
-                self.license_files[pkg.name]['files'].append(file)
-
+            if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                return PipelineResult.STOP
         if gaction == GlobalAction.LICENSES:
-            return True
+            return PipelineResult.CONTINUE
         if paction == PkgAction.LICENSE and pkg.name == target:
-            return False
+            return PipelineResult.STOP
 
         # load any late-stage configuration options from the remote
         # sources
@@ -176,75 +204,76 @@ class RelengPackagePipeline:
             # custom execution command
             if paction == PkgAction.EXEC and pkg.name == target:
                 self._stage_exec(pkg)
-                return False
+                return PipelineResult.STOP
 
             # bootstrapping
-            flag = pkg._ff_bootstrap
-            if check_file_flag(flag) == FileFlag.NO_EXIST:
+            fflag = pkg._ff_bootstrap
+            if check_file_flag(fflag) == FileFlag.NO_EXIST:
                 self.engine.stats.track_duration_start(pkg.name, 'boot')
                 if not bootstrap_stage(self.engine, pkg, pkg_env):
                     raise RelengToolBootstrapStageFailure
                 self.engine.stats.track_duration_end(pkg.name, 'boot')
-                if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                    return False
+                if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                    return PipelineResult.STOP
 
             # configuring
-            flag = pkg._ff_configure
-            if check_file_flag(flag) == FileFlag.NO_EXIST:
+            fflag = pkg._ff_configure
+            if check_file_flag(fflag) == FileFlag.NO_EXIST:
                 self.engine.stats.track_duration_start(pkg.name, 'configure')
                 if not configure_stage(self.engine, pkg, pkg_env):
                     raise RelengToolConfigurationStageFailure
                 self.engine.stats.track_duration_end(pkg.name, 'configure')
-                if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                    return False
+                if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                    return PipelineResult.STOP
             if paction in (PkgAction.CONFIGURE, PkgAction.RECONFIGURE_ONLY):
                 if pkg.name == target:
-                    return False
+                    return PipelineResult.STOP
 
             # building
-            flag = pkg._ff_build
-            if check_file_flag(flag) == FileFlag.NO_EXIST:
+            fflag = pkg._ff_build
+            if check_file_flag(fflag) == FileFlag.NO_EXIST:
                 self.engine.stats.track_duration_start(pkg.name, 'build')
                 if not build_stage(self.engine, pkg, pkg_env):
                     raise RelengToolBuildStageFailure
                 self.engine.stats.track_duration_end(pkg.name, 'build')
-                if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                    return False
+                if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                    return PipelineResult.STOP
             if paction in (PkgAction.BUILD, PkgAction.REBUILD_ONLY):
                 if pkg.name == target:
-                    return False
+                    return PipelineResult.STOP
 
             # installing
-            flag = pkg._ff_install
-            if check_file_flag(flag) == FileFlag.NO_EXIST:
+            fflag = pkg._ff_install
+            if check_file_flag(fflag) == FileFlag.NO_EXIST:
                 self.engine.stats.track_duration_start(pkg.name, 'install')
                 if not install_stage(self.engine, pkg, pkg_env):
                     raise RelengToolInstallStageFailure
                 self.engine.stats.track_duration_end(pkg.name, 'install')
-                if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                    return False
+                if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                    return PipelineResult.STOP
             # (note: re-install requests will re-invoke package-specific
             # post-processing)
 
             # package-specific post-processing
-            flag = pkg._ff_post
-            if check_file_flag(flag) == FileFlag.NO_EXIST:
+            fflag = pkg._ff_post
+            if check_file_flag(fflag) == FileFlag.NO_EXIST:
                 self.engine.stats.track_duration_start(pkg.name, 'post')
                 if not post_stage(self.engine, pkg, pkg_env):
                     raise RelengToolPostStageFailure
                 self.engine.stats.track_duration_end(pkg.name, 'post')
-                if process_file_flag(True, flag) != FileFlag.CONFIGURED:
-                    return False
+                if process_file_flag(fflag, flag=True) != FileFlag.CONFIGURED:
+                    return PipelineResult.STOP
             if paction in (
                     PkgAction.INSTALL,
                     PkgAction.REBUILD_ONLY,
                     PkgAction.RECONFIGURE_ONLY,
                     PkgAction.REINSTALL):
                 if pkg.name == target:
-                    return False
+                    return PipelineResult.STOP
 
-        return True
+        return PipelineResult.CONTINUE
 
+    @contextmanager
     def _stage_env(self, pkg):
         """
         prepare environment variables for a specific package processing
@@ -257,10 +286,30 @@ class RelengPackagePipeline:
         Args:
             pkg: the package being processed
 
-        Returns:
+        Yields:
             the prepared package-enhanced environment variables
         """
 
+        pkg_keys = [
+            'PKG_BUILD_BASE_DIR',
+            'PKG_BUILD_DIR',
+            'PKG_BUILD_OUTPUT_DIR',
+            'PKG_CACHE_DIR',
+            'PKG_CACHE_FILE',
+            'PKG_DEFDIR',
+            'PKG_DEVMODE',
+            'PKG_INTERNAL',
+            'PKG_LOCALSRCS',
+            'PKG_NAME',
+            'PKG_REVISION',
+            'PKG_SITE',
+            'PKG_VERSION',
+        ]
+
+        saved_env = {}
+        for key in pkg_keys:
+            saved_env[key] = os.environ.get(key, None)
+
         # copy environment since packages do not share values
         pkg_env = self.script_env.copy()
 
@@ -271,24 +320,39 @@ class RelengPackagePipeline:
 
         # always register optional flags in script environment
         pkg_env['PKG_INTERNAL'] = None
+        pkg_env['PKG_DEVMODE'] = None
+        pkg_env['PKG_LOCALSRCS'] = None
 
-        # package variables
-        for env in (os.environ, pkg_env):
-            env['PKG_BUILD_BASE_DIR'] = pkg.build_dir
-            env['PKG_BUILD_DIR'] = build_dir
-            env['PKG_BUILD_OUTPUT_DIR'] = pkg.build_output_dir
-            env['PKG_CACHE_DIR'] = pkg.cache_dir
-            env['PKG_CACHE_FILE'] = pkg.cache_file
-            env['PKG_DEFDIR'] = pkg.def_dir
-            env['PKG_NAME'] = pkg.name
-            env['PKG_SITE'] = pkg.site if pkg.site else ''
-            env['PKG_REVISION'] = pkg.revision
-            env['PKG_VERSION'] = pkg.version
+        try:
+            for env in (os.environ, pkg_env):
+                env['PKG_BUILD_BASE_DIR'] = pkg.build_dir
+                env['PKG_BUILD_DIR'] = build_dir
+                env['PKG_BUILD_OUTPUT_DIR'] = pkg.build_output_dir
+                env['PKG_CACHE_DIR'] = pkg.cache_dir
+                env['PKG_CACHE_FILE'] = pkg.cache_file
+                env['PKG_DEFDIR'] = pkg.def_dir
+                env['PKG_NAME'] = pkg.name
+                env['PKG_SITE'] = pkg.site if pkg.site else ''
+                env['PKG_REVISION'] = pkg.revision
+                env['PKG_VERSION'] = pkg.version
 
-            if pkg.is_internal:
-                env['PKG_INTERNAL'] = '1'
+                if pkg.devmode:
+                    env['PKG_DEVMODE'] = '1'
 
-        return pkg_env
+                if pkg.is_internal:
+                    env['PKG_INTERNAL'] = '1'
+
+                if pkg.local_srcs:
+                    env['PKG_LOCALSRCS'] = '1'
+
+            yield pkg_env
+        finally:
+            # restore any overrides that may have been set
+            for k, v in saved_env.items():
+                if v is not None:
+                    os.environ[k] = v
+                else:
+                    os.environ.pop(k, None)
 
     @contextmanager
     def _stage_env_finalize(self, pkg, pkg_env):
@@ -303,9 +367,13 @@ class RelengPackagePipeline:
         Args:
             pkg: the package being processed
             pkg_env: the environment to populate
+
+        Yields:
+            the prepared package-enhanced environment variables
         """
 
         pkg_keys = [
+            'HOST_BIN_DIR',
             'HOST_INCLUDE_DIR',
             'HOST_LIB_DIR',
             'NJOBS',
@@ -314,8 +382,10 @@ class RelengPackagePipeline:
             'PREFIXED_HOST_DIR',
             'PREFIXED_STAGING_DIR',
             'PREFIXED_TARGET_DIR',
+            'STAGING_BIN_DIR',
             'STAGING_INCLUDE_DIR',
             'STAGING_LIB_DIR',
+            'TARGET_BIN_DIR',
             'TARGET_INCLUDE_DIR',
             'TARGET_LIB_DIR',
         ]
@@ -334,22 +404,28 @@ class RelengPackagePipeline:
                     staging_pdir = self.opts.staging_dir + nprefix
                     target_pdir = self.opts.target_dir + nprefix
 
+                    host_bin_dir = os.path.join(host_pdir, 'bin')
                     host_include_dir = os.path.join(host_pdir, 'include')
                     host_lib_dir = os.path.join(host_pdir, 'lib')
+                    staging_bin_dir = os.path.join(staging_pdir, 'bin')
                     staging_include_dir = os.path.join(staging_pdir, 'include')
                     staging_lib_dir = os.path.join(staging_pdir, 'lib')
+                    target_bin_dir = os.path.join(target_pdir, 'bin')
                     target_include_dir = os.path.join(target_pdir, 'include')
                     target_lib_dir = os.path.join(target_pdir, 'lib')
 
                     # will override existing prefix related variables
+                    env['HOST_BIN_DIR'] = host_bin_dir
                     env['HOST_INCLUDE_DIR'] = host_include_dir
                     env['HOST_LIB_DIR'] = host_lib_dir
                     env['PREFIX'] = pkg.prefix
                     env['PREFIXED_HOST_DIR'] = host_pdir
                     env['PREFIXED_STAGING_DIR'] = staging_pdir
                     env['PREFIXED_TARGET_DIR'] = target_pdir
+                    env['STAGING_BIN_DIR'] = staging_bin_dir
                     env['STAGING_INCLUDE_DIR'] = staging_include_dir
                     env['STAGING_LIB_DIR'] = staging_lib_dir
+                    env['TARGET_BIN_DIR'] = target_bin_dir
                     env['TARGET_INCLUDE_DIR'] = target_include_dir
                     env['TARGET_LIB_DIR'] = target_lib_dir
 
