@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 # A flag (or option) can be driven by the existence of a file (i.e. a file
 # flag). When an instance operates with file flags, two modes are approached in
@@ -15,6 +16,7 @@ from releng_tool.util.io import touch
 from releng_tool.util.log import err
 import os
 
+
 class FileFlag(Enum):
     """
     file flag result states
@@ -30,6 +32,7 @@ class FileFlag(Enum):
     NOT_CONFIGURED = 'not_configured'
     NO_EXIST = 'no_exist'
 
+
 def check_file_flag(file):
     """
     check a file flag
@@ -43,9 +46,10 @@ def check_file_flag(file):
         ``FileFlag.EXISTS`` if the flag is enabled; ``FileFlag.NO_EXIST`` if the
             flag is not enabled
     """
-    return process_file_flag(None, file)
+    return process_file_flag(file, None)
 
-def process_file_flag(flag, file, quiet=False):
+
+def process_file_flag(file, flag, quiet=False):
     """
     process a file flag event
 
@@ -57,8 +61,8 @@ def process_file_flag(flag, file, quiet=False):
     not the flag is considered enabled.
 
     Args:
-        flag: the flag option to used; ``None`` to check flag state
         file: the filename
+        flag: the flag option to used; ``None`` to check flag state
         quiet: suppression of any error messages to standard out
 
     Returns:
