@@ -1,20 +1,16 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 releng-tool
+# Copyright 2018-2023 releng-tool
+# SPDX-License-Identifier: BSD-2-Clause
 
 from __future__ import unicode_literals
 from contextlib import contextmanager
-from releng_tool.util.compat import CopyTreeError
-from releng_tool.util.compat import copy_tree
 from releng_tool.util.log import debug
 from releng_tool.util.log import err
 from releng_tool.util.log import is_verbose
 from releng_tool.util.log import verbose
 from releng_tool.util.log import warn
 from runpy import run_path
-from shutil import Error as ShutilError
-from shutil import copy2
 from shutil import copyfileobj
-from shutil import move
 import errno
 import os
 import stat
@@ -39,16 +35,25 @@ MULTIPART_EXTENSIONS = [
     'tar.z',
 ]
 
+# file extensions permitted for releng-tool scripts/definitions
+RELENG_TOOL_EXTENSIONS = [
+    '.releng',
+    '.py',
+]
+
+
 class FailedToPrepareBaseDirectoryError(Exception):
     """
     raised when a base directory could not be prepared
     """
 
+
 class FailedToPrepareWorkingDirectoryError(Exception):
     """
     raised when a working directory could not be prepared
     """
 
+
 def cat(file, *args):
     """
     concatenate files and print on the standard output
@@ -89,7 +94,8 @@ def cat(file, *args):
     except OSError:
         return False
 
-def ensure_dir_exists(dir_, quiet=False):
+
+def ensure_dir_exists(dir_, quiet=False, critical=False):
     """
     ensure the provided directory exists
 
@@ -112,6 +118,8 @@ def ensure_dir_exists(dir_, quiet=False):
         dir_: the directory
         quiet (optional): whether or not to suppress output (defaults to
             ``False``)
+        critical (optional): whether or not to stop execution on failure
+            (defaults to ``False``)
 
     Returns:
         ``True`` if the directory exists; ``False`` if the directory could not
@@ -124,9 +132,12 @@ def ensure_dir_exists(dir_, quiet=False):
             if not quiet:
                 err('unable to create directory: {}\n'
                     '    {}', dir_, e)
+            if critical:
+                sys.exit(-1)
             return False
     return True
 
+
 def execute(args, cwd=None, env=None, env_update=None, quiet=None,
         critical=True, poll=False, capture=None):
     """
@@ -207,6 +218,7 @@ def execute(args, cwd=None, env=None, env_update=None, quiet=None,
     )
     return rv == 0
 
+
 def execute_rv(command, *args, **kwargs):
     """
     execute the provided command/arguments
@@ -259,6 +271,7 @@ def execute_rv(command, *args, **kwargs):
     )
     return rv, '\n'.join(out)
 
+
 def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
         critical=True, poll=False, capture=None):
     """
@@ -347,10 +360,10 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
 
         # python 2.7 can have trouble with unicode environment variables;
         # forcing all values to an ascii type
-        if final_env and sys.version_info[0] < 3:
+        if final_env and sys.version_info[0] < 3:  # noqa: PLR2004
             debug('detected python 2.7; sanity checking environment variables')
             for k, v in final_env.items():
-                if isinstance(v, unicode): # pylint: disable=E0602 # noqa: F821
+                if isinstance(v, unicode):  # pylint: disable=E0602 # noqa: F821
                     final_env[k] = v.encode('ascii', 'replace')
 
         if is_verbose():
@@ -388,9 +401,9 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
                     if not c and proc.poll() is not None:
                         break
                     line += c
-                    if c == b'\r' or c == b'\n':
+                    if c == b'\r' or c == b'\n':  # noqa: PLR2004
                         decoded_line = line.decode('utf_8')
-                        if c == b'\n' and capture is not None:
+                        if c == b'\n' and capture is not None:  # noqa: PLR2004
                             capture.append(decoded_line)
                         if not quiet:
                             sys.stdout.write(decoded_line)
@@ -418,10 +431,7 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
 
     if rv != 0:
         if critical:
-            if args:
-                cmd_str = _cmd_args_to_str(args)
-            else:
-                cmd_str = '<empty>'
+            cmd_str = _cmd_args_to_str(args) if args else '<empty>'
             err('failed to issue command: ' + cmd_str)
 
             # trigger a hard stop
@@ -433,6 +443,7 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
 
     return rv
 
+
 def _cmd_args_to_str(args):
     """
     convert an argument list to a platform escaped string
@@ -460,6 +471,7 @@ def _cmd_args_to_str(args):
 
     return cmd_str
 
+
 @contextmanager
 def generate_temp_dir(dir_=None):
     """
@@ -499,6 +511,7 @@ def generate_temp_dir(dir_=None):
                 warn('unable to cleanup temporary directory: {}\n'
                      '    {}', dir_, e)
 
+
 @contextmanager
 def interim_working_dir(dir_):
     """
@@ -540,6 +553,7 @@ def interim_working_dir(dir_):
         except IOError:
             warn('unable to restore original working directory: ' + owd)
 
+
 def interpret_stem_extension(basename):
     """
     interpret the stem and extension from a provided basename
@@ -580,6 +594,7 @@ def interpret_stem_extension(basename):
 
     return stem, ext
 
+
 def ls(dir_):
     """
     list a directory's contents
@@ -616,20 +631,21 @@ def ls(dir_):
     except OSError:
         return False
 
+
 def opt_file(file):
     """
     return a file (and existence) to opt for based a given file path
 
     Various user-defined scripts by default do not have an extension. For
     example, a project's releng-tool script is defined by a file named `releng`;
-    however, select users may wish to define a `releng.py` script instead.
-    Consider, with the previous example, the file `releng` is the "standard"
-    file and the `releng.py` script is the "alternative" file. This utility call
-    will return the file path and the existence state of the returned file. If
-    the standard file does not exist but the alternative file does, this call
-    will return the alternative file. Priority is given to the standard file, so
-    if neither file exists, this call will return the provided/standard file
-    path.
+    however, select users may wish to define a `releng.releng` or `releng.py`
+    script instead. Consider, with the previous example, the file `releng` is
+    the "standard" file and the `releng.releng` or `releng.py` script is the
+    "alternative" file. This utility call will return the file path and the
+    existence state of the returned file. If the standard file does not exist
+    but the alternative file does, this call will return the alternative file.
+    Priority is given to the standard file, so if neither file exists, this
+    call will return the provided/standard file path.
 
     Args:
         file: the file to check for
@@ -640,96 +656,13 @@ def opt_file(file):
 
     exists = os.path.isfile(file)
     if not exists:
-        flex_file = file + '.py'
-        if os.path.isfile(flex_file):
-            return flex_file, True
+        for ext in RELENG_TOOL_EXTENSIONS:
+            flex_file = file + ext
+            if os.path.isfile(flex_file):
+                return flex_file, True
 
     return file, exists
 
-def path_copy(src, dst, quiet=False, critical=True, dst_dir=False):
-    """
-    copy a file or directory into a target file or directory
-
-    This call will attempt to copy a provided file or directory, defined by
-    ``src`` into a destination file or directory defined by ``dst``. If ``src``
-    is a file, then ``dst`` is considered to be a file or directory; if ``src``
-    is a directory, ``dst`` is considered a target directory. If a target
-    directory or target file's directory does not exist, it will be
-    automatically created. In the event that a file or directory could not be
-    copied, an error message will be output to standard error (unless ``quiet``
-    is set to ``True``). If ``critical`` is set to ``True`` and the specified
-    file/directory could not be copied for any reason, this call will issue a
-    system exit (``SystemExit``).
-
-    An example when using in the context of script helpers is as follows:
-
-    .. code-block:: python
-
-        # (stage)
-        # my-file
-        releng_copy('my-file', 'my-file2')
-        # (stage)
-        # my-file
-        # my-file2
-        releng_copy('my-file', 'my-directory/')
-        # (stage)
-        # my-directory/my-file
-        # my-file
-        # my-file2
-        releng_copy('my-directory/', 'my-directory2/')
-        # (stage)
-        # my-directory/my-file
-        # my-directory2/my-file
-        # my-file
-        # my-file2
-
-    Args:
-        src: the source directory or file
-        dst: the destination directory or file\\* (\\*if ``src`` is a file)
-        quiet (optional): whether or not to suppress output
-        critical (optional): whether or not to stop execution on failure
-        dst_dir (optional): force hint that the destination is a directory
-
-    Returns:
-        ``True`` if the copy has completed with no error; ``False`` if the copy
-        has failed
-
-    Raises:
-        SystemExit: if the copy operation fails with ``critical=True``
-    """
-    success = False
-    errmsg = None
-
-    try:
-        if os.path.isfile(src):
-            attempt_copy = True
-
-            if dst_dir:
-                base_dir = dst
-            else:
-                base_dir = os.path.dirname(dst)
-
-            if base_dir and not os.path.isdir(base_dir):
-                attempt_copy = ensure_dir_exists(base_dir, quiet=quiet)
-
-            if attempt_copy:
-                copy2(src, dst)
-                success = True
-        elif os.path.exists(src):
-            copy_tree(src, dst)
-            success = True
-        else:
-            errmsg = 'source does not exist: {}'.format(src)
-    except (CopyTreeError, IOError, ShutilError) as e:
-        errmsg = str(e)
-
-    if not quiet and errmsg:
-        err('unable to copy source contents to target location\n'
-            '    {}', errmsg)
-
-    if not success and critical:
-        sys.exit(-1)
-    return success
 
 def path_exists(path, *args):
     """
@@ -756,150 +689,6 @@ def path_exists(path, *args):
     """
     return os.path.exists(os.path.join(path, *args))
 
-def path_move(src, dst, quiet=False, critical=True):
-    """
-    move a file or directory into a target file or directory
-
-    This call will attempt to move a provided file or directory's contents,
-    defined by ``src`` into a destination file or directory defined by ``dst``.
-    If ``src`` is a file, then ``dst`` is considered to be a file or directory;
-    if ``src`` is a directory, ``dst`` is considered a target directory. If a
-    target directory or target file's directory does not exist, it will be
-    automatically created.
-
-    In the event that a file or directory could not be moved, an error message
-    will be output to standard error (unless ``quiet`` is set to ``True``). If
-    ``critical`` is set to ``True`` and the specified file/directory could not
-    be moved for any reason, this call will issue a system exit
-    (``SystemExit``).
-
-    An example when using in the context of script helpers is as follows:
-
-    .. code-block:: python
-
-        # (input)
-        # my-directory/another-file
-        # my-file
-        # my-file2
-        releng_move('my-file', 'my-file3')
-        releng_move('my-directory/', 'my-directory2/')
-        releng_move('my-file2', 'my-directory2/')
-        # (output)
-        # my-directory2/another-file
-        # my-directory2/my-file2
-        # my-file3
-
-    Args:
-        src: the source directory or file
-        dst: the destination directory or file\\* (\\*if ``src`` is a file)
-        quiet (optional): whether or not to suppress output
-        critical (optional): whether or not to stop execution on failure
-
-    Returns:
-        ``True`` if the move has completed with no error; ``False`` if the move
-        has failed
-
-    Raises:
-        SystemExit: if the copy operation fails with ``critical=True``
-    """
-    success = True
-
-    if src == dst:
-        return True
-
-    if os.path.isfile(src):
-        parent_dir = os.path.dirname(dst)
-        if parent_dir and not os.path.isdir(parent_dir):
-            success = ensure_dir_exists(parent_dir, quiet=quiet)
-    elif not os.path.isdir(dst):
-        if os.path.exists(dst):
-            _path_remove_file(dst)
-
-        success = ensure_dir_exists(dst, quiet=quiet)
-    else:
-        src_dir = os.path.realpath(src)
-        dst_dir = os.path.realpath(dst)
-        if dst_dir.startswith(src_dir):
-            if not quiet:
-                err('unable to move source contents to target location\n'
-                    '    attempt to move directory into a child subdirectory')
-            if critical:
-                sys.exit(-1)
-            return False
-
-    if success:
-        try:
-            if os.path.isfile(src):
-                if os.path.isfile(dst):
-                    _path_remove_file(dst)
-
-                move(src, dst)
-            else:
-                _path_move(src, dst)
-        except Exception as e:
-            success = False
-            if not quiet:
-                err('unable to move source contents to target location\n'
-                    '    {}', e)
-
-    if not success and critical:
-        sys.exit(-1)
-    return success
-
-def _path_move(src, dst):
-    """
-    move the provided directory into the target directory (recursive)
-
-    Attempts to move the provided directory into the target directory. In the
-    event that a file or directory could not be moved due to an error, this
-    function will typically raise an OSError exception for `pathMove` to handle.
-
-    In the chance that a file cannot be moved due to permission issues, this
-    function can attempt to adjust permissions to specific paths to help in
-    the moving processes (e.g. dealing with read-only files or other strict
-    permissions setup during a build process).
-
-    Args:
-        src: the source directory
-        dst: the destination directory
-
-    Raises:
-        OSError: if a path could not be moved
-    """
-
-    # ensure a caller has read/write access before hand to prepare for moving
-    # (e.g. if marked as read-only) and ensure contents can be fetched as well
-    try:
-        st = os.stat(src)
-        if not (st.st_mode & stat.S_IRUSR) or not (st.st_mode & stat.S_IWUSR):
-            os.chmod(src, st.st_mode | stat.S_IRUSR | stat.S_IWUSR)
-    except OSError:
-        pass
-
-    entries = os.listdir(src)
-    for entry in entries:
-        src_path = os.path.join(src, entry)
-        dst_path = os.path.join(dst, entry)
-
-        if os.path.isdir(src_path) and not os.path.islink(src_path):
-            if os.path.isdir(dst_path):
-                _path_move(src_path, dst_path)
-            else:
-                if os.path.exists(dst_path):
-                    _path_remove_file(dst_path)
-
-                move(src_path, dst_path)
-        else:
-            if os.path.exists(dst_path):
-                if os.path.isdir(dst_path) and not os.path.islink(dst_path):
-                    _path_remove_dir(dst_path)
-                else:
-                    _path_remove_file(dst_path)
-
-            move(src_path, dst_path)
-
-    # remove directory
-    os.rmdir(src)
 
 def path_remove(path, quiet=False):
     """
@@ -945,6 +734,7 @@ def path_remove(path, quiet=False):
 
     return True
 
+
 def _path_remove_dir(dir_):
     """
     remove the provided directory (recursive)
@@ -986,6 +776,7 @@ def _path_remove_dir(dir_):
     # remove directory
     os.rmdir(dir_)
 
+
 def _path_remove_file(path):
     """
     remove the provided file
@@ -1024,6 +815,7 @@ def _path_remove_file(path):
         except OSError:
             raise e
 
+
 def prepare_arguments(args):
     """
     prepares arguments from a dictionary
@@ -1053,6 +845,7 @@ def prepare_arguments(args):
 
     return final
 
+
 def prepare_definitions(defs, prefix=None):
     """
     prepares definitions from a dictionary
@@ -1082,13 +875,11 @@ def prepare_definitions(defs, prefix=None):
             if prefix:
                 key = prefix + key
 
-            if val:
-                final.append('{}={}'.format(key, val))
-            else:
-                final.append(key)
+            final.append('{}={}'.format(key, val))
 
     return final
 
+
 def prepend_shebang_interpreter(args):
     """
     prepend interpreter program (if any) to argument list
@@ -1108,7 +899,7 @@ def prepend_shebang_interpreter(args):
     """
     try:
         with open(args[0], 'rb') as f:
-            if f.read(1) == b'#' and f.read(1) == b'!':
+            if f.read(1) == b'#' and f.read(1) == b'!':  # noqa: PLR2004
                 MAXINTERP = 2048
                 interp = f.readline(MAXINTERP + 1).rstrip()
                 if len(interp) > MAXINTERP:
@@ -1119,6 +910,7 @@ def prepend_shebang_interpreter(args):
         pass
     return args
 
+
 def run_script(script, globals_, subject=None, catch=True):
     """
     execute the provided script and provide the resulting globals module
@@ -1159,6 +951,7 @@ def run_script(script, globals_, subject=None, catch=True):
 
     return result
 
+
 def touch(file):
     """
     update a file's access/modifications times
